@@ -32,10 +32,10 @@ BinHeader::BinHeader() {
 
 uchar* BinHeader::insert(QString s, uchar* p) {
     int strlen=s.length();
-    int suint = sizeof(strlen);
-    memcpy(p, &strlen, suint);
+//     int suint = sizeof(strlen);
+    memcpy(p, &strlen, szInt);
 
-    p+=suint;
+	p+=szInt;
     memcpy(p, (const char *)s, strlen);
     p+=strlen;
     return p;
@@ -48,20 +48,22 @@ uchar* BinHeader::insert(QString s, uchar* p) {
 // Retrieves the string at current position in array i into the QString s
 // updates and returns i;
 
-uchar* BinHeader::retrieve(uchar* i,QString &s) {
+uchar* BinHeader::retrieve(uchar* i,QString &s) 
+{
 
     int strlen;
-    int ssize=sizeof(strlen);
+//     int ssize=sizeof(strlen);
 
     char *temp;
-    memcpy(&strlen, i, ssize);
+    memcpy(&strlen, i, szInt);
 
-    i+=ssize;
+    i+=szInt;
     temp=new char[strlen+1];
     memcpy(temp, i, strlen);
     temp[strlen]='\0';
+	//double memcopy...save strings with NULL termination and do a "s=i"?
     s=temp;
-    delete temp;
+    delete [] temp;
     i+=strlen;
     return i;
 
@@ -71,7 +73,8 @@ uchar* BinHeader::retrieve(uchar* i,QString &s) {
 
 // Marshall the binHeader
 
-uchar *BinHeader::data() {
+uchar *BinHeader::data() 
+{
 
 
 
@@ -79,7 +82,7 @@ uchar *BinHeader::data() {
     uchar *i=&p[0];
 	//Couples serverid-article number (at least 1)
 	int part, serverNumItems, serverId, artNum;
-	int szInt=sizeof(uint);
+// 	int szInt=sizeof(uint);
 	serverNumItems=serverLowest.count();
 	memcpy(i, &serverNumItems, szInt);
 	i+=szInt;
@@ -123,8 +126,23 @@ uchar *BinHeader::data() {
     //size field
     memcpy(i, &size, szInt);
     i+=szInt;
+	
+	//INSERTED SERVERPARTITEMS BETWENN SIZE AND PARTNUMITEMS
+	
+	int serverPartItems=serverPart.count();
 
-
+	memcpy(i, &serverPartItems, szInt);
+	i+=szInt;
+	for (spmit=serverPart.begin() ; spmit != serverPart.end(); ++spmit) {
+		serverId=spmit.key();
+		artNum=spmit.data();
+		memcpy(i, &serverId, szInt);
+		i+=szInt;
+		memcpy(i, &artNum, szInt);
+		i+=szInt;
+	}
+	
+	
     //how many items in the map?
     int partNumItems=partNum.count();
     memcpy(i, &partNumItems , szInt);
@@ -170,19 +188,8 @@ uchar *BinHeader::data() {
 		
 	
 	//END NEW MID
-    // save servernummap count
-    int serverPartItems=serverPart.count();
-
-    memcpy(i, &serverPartItems, szInt);
-    i+=szInt;
-    for (spmit=serverPart.begin() ; spmit != serverPart.end(); ++spmit) {
-        serverId=spmit.key();
-        artNum=spmit.data();
-        memcpy(i, &serverId, szInt);
-        i+=szInt;
-        memcpy(i, &artNum, szInt);
-        i+=szInt;
-    }
+    
+    
 	
 	//Using serverPartitems for partSize
 	serverPartItems=partSize.count();
@@ -200,28 +207,24 @@ uchar *BinHeader::data() {
 		i+=szInt;
 		
 	}
-			
-
-
-
-
-
-
+	
+	//Save the length...
+	serverPartItems = i-p+szInt;
+// 	qDebug("binheader size: %d", serverPartItems);
+	
+	memcpy(i,&serverPartItems , szInt);
 
     return p;
-
-
-
-
 }
 
 //create binheader, unmarshalling DB data
 
-BinHeader::BinHeader(uchar *p, int version) {
+BinHeader::BinHeader(uchar *p, int version) 
+{
 
 
     uchar *pi =&p[0];
-	int szInt=sizeof(int);
+// 	int szInt=sizeof(int);
 	uint  part, serverNumItems, serverId, artNum;
 	//BEGIN serverLowest code
 	if (version >= 3) {
@@ -247,7 +250,6 @@ BinHeader::BinHeader(uchar *p, int version) {
 
 
     // Fixed-size fields:
-    
 
     memcpy(&numParts, pi, szInt);
     pi+=szInt;
@@ -260,35 +262,51 @@ BinHeader::BinHeader(uchar *p, int version) {
     pi+=szInt;
     memcpy(&size, pi, szInt);
     pi+=szInt;
-
-    uint partNumItems;
-    memcpy(&partNumItems, pi, szInt);
-    pi+=szInt;
+	
+	uint serverPartItems, partNumItems;
+	if (version >= 4) {
+		memcpy(&serverPartItems, pi, szInt);
+		pi+=szInt;
+		for (uint i =0; i< serverPartItems; i++) {
+	
+			memcpy(&serverId, pi, szInt);
+			pi+=szInt;
+			memcpy(&artNum, pi, szInt);
+			pi+=szInt;
+			serverPart[serverId]=artNum;
+	
+		}
+	}
+		
+	memcpy(&partNumItems, pi, szInt);
+	pi+=szInt;
 
 
-    
-    // mapItems
-    //     memcpy(&mapItems, pi, szInt);
-    //     pi+=szInt;
+	
+	// mapItems
+	//     memcpy(&mapItems, pi, szInt);
+	//     pi+=szInt;
 
-    for (uint i = 0; i < partNumItems; i++) {
-        // get the partid;
-        memcpy(&part, pi, szInt);
-        pi+=szInt;
-        //get the number of items in the servernum map
-        memcpy(&serverNumItems, pi, szInt);
-        pi+=szInt;
-        // now rebuild the servernum map
-        // 		partNum.insert(part);
-        for (uint j = 0; j < serverNumItems; j++) {
-            memcpy(&serverId, pi, szInt);
-            pi+=szInt;
-            memcpy(&artNum, pi, szInt);
-            pi+=szInt;
-            partNum[part].insert(serverId, artNum);
-        }
-    }
-	//BEGIN partMid code
+	for (uint i = 0; i < partNumItems; i++) {
+		// get the partid;
+		memcpy(&part, pi, szInt);
+		pi+=szInt;
+		//get the number of items in the servernum map
+		memcpy(&serverNumItems, pi, szInt);
+		pi+=szInt;
+		// now rebuild the servernum map
+		// 		partNum.insert(part);
+		for (uint j = 0; j < serverNumItems; j++) {
+			memcpy(&serverId, pi, szInt);
+			pi+=szInt;
+			memcpy(&artNum, pi, szInt);
+			pi+=szInt;
+			partNum[part].insert(serverId, artNum);
+		}
+	}
+	
+	
+		//BEGIN partMid code
 		
 	if (version >= 2) {
 		uint partMidItems;
@@ -305,19 +323,21 @@ BinHeader::BinHeader(uchar *p, int version) {
 	
 	}
 	//END partMid code
-
-    uint serverPartItems;
-    memcpy(&serverPartItems, pi, szInt);
-    pi+=szInt;
-    for (uint i =0; i< serverPartItems; i++) {
-
-        memcpy(&serverId, pi, szInt);
-        pi+=szInt;
-        memcpy(&artNum, pi, szInt);
-        pi+=szInt;
-        serverPart[serverId]=artNum;
-
-    }
+	
+	if (version < 4) {
+	
+		memcpy(&serverPartItems, pi, szInt);
+		pi+=szInt;
+		for (uint i =0; i< serverPartItems; i++) {
+	
+			memcpy(&serverId, pi, szInt);
+			pi+=szInt;
+			memcpy(&artNum, pi, szInt);
+			pi+=szInt;
+			serverPart[serverId]=artNum;
+	
+		}
+	}
 	
 	//ok, if version >= 1 we have the size of the single parts to consider...
 	//Using serverPartItems, serverId, artnum to limit autovariables allocation...
@@ -336,26 +356,124 @@ BinHeader::BinHeader(uchar *p, int version) {
 		}
 		
 	}
+	
+}
+
+BinHeader::BinHeader(uchar *p) 
+{
+	uchar *pi =&p[0];
+	uint  part, serverNumItems, serverId, artNum;
+
+	//Reload serverId, <lowest article number>
+	int serverLowestItems;
+	memcpy(&serverLowestItems, pi, szInt);
+	pi+=szInt;
+	for (int i = 0; i < serverLowestItems; i++) {
+		memcpy(&serverId, pi, szInt);
+		pi+=szInt;
+		memcpy(&part, pi, szInt);
+		pi+=szInt;
+		serverLowest.insert(serverId, part);
+	}
+
+    // Variable-lenght fields
+
+	pi=retrieve(pi,subj);
+	pi=retrieve(pi,from);
+	pi=retrieve(pi,date);
+
+    // Fixed-size fields:
+
+	memcpy(&numParts, pi, szInt);
+	pi+=szInt;
+	memcpy(&missingParts, pi, szInt);
+	pi+=szInt;
+	memcpy(&status, pi, szInt);
+	pi+=szInt;
+    //status
+	memcpy(&lines, pi, szInt);
+	pi+=szInt;
+	memcpy(&size, pi, szInt);
+	pi+=szInt;
+	
+	uint serverPartItems, partNumItems;
+
+	memcpy(&serverPartItems, pi, szInt);
+	pi+=szInt;
+	for (uint i =0; i< serverPartItems; i++) {
+
+		memcpy(&serverId, pi, szInt);
+		pi+=szInt;
+		memcpy(&artNum, pi, szInt);
+		pi+=szInt;
+		serverPart[serverId]=artNum;
+
+	}
+		
+	memcpy(&partNumItems, pi, szInt);
+	pi+=szInt;
 
 
+	for (uint i = 0; i < partNumItems; i++) {
+		// get the partid;
+		memcpy(&part, pi, szInt);
+		pi+=szInt;
+		//get the number of items in the servernum map
+		memcpy(&serverNumItems, pi, szInt);
+		pi+=szInt;
+		// now rebuild the servernum map
+		// 		partNum.insert(part);
+		for (uint j = 0; j < serverNumItems; j++) {
+			memcpy(&serverId, pi, szInt);
+			pi+=szInt;
+			memcpy(&artNum, pi, szInt);
+			pi+=szInt;
+			partNum[part].insert(serverId, artNum);
+		}
+	}
+		
+	
+	uint partMidItems;
+	memcpy(&partMidItems, pi, szInt);
+	pi+=szInt;
+	QString mid;
+	for (uint i = 0; i < partMidItems; i++) {
+		memcpy(&part, pi, szInt);
+		pi+=szInt;
+		pi = retrieve(pi, mid);
+		partMid[part]=mid;
+		
+	}
 
-
-
-
-
-
+	
+// 		qDebug("Loading size into bh");
+	memcpy(&serverPartItems, pi, szInt);
+	pi+=szInt;
+	for (uint i = 0; i < serverPartItems; i++) {
+		memcpy(&serverId, pi, szInt);
+		pi+=szInt;
+		memcpy(&artNum, pi, szInt);
+		pi+=szInt;
+		partSize[serverId]=artNum;
+	}
 
 }
 
+
+
 //Updated to accomodate part sizes...
 
 uint BinHeader::getRecordSize( ) {
     uint partNumSize=0;
     for (pnmit=partNum.begin(); pnmit != partNum.end(); ++pnmit) {
-        partNumSize+=(pnmit.data().count()*2*sizeof(uint))+sizeof(uint);
-        partNumSize+=sizeof(uint);
+		//every element is a couple of int. So I need space 
+		//for the count (number of couples in the part) and the couple itself
+		partNumSize+=(pnmit.data().count()*2*szInt)+szInt; 
+		// space for the part #
+        partNumSize+=szInt;
     }
-    partNumSize+=sizeof(uint);
+	//space for the number of parts
+    partNumSize+=szInt;
 
 	/* For every part-mid couple, we have:
 		- A int which holds the number of parts
@@ -368,7 +486,7 @@ uint BinHeader::getRecordSize( ) {
 		
 	*/
 	PartMid::Iterator partMidIt;
-	uint partMidSize = 2*sizeof(int)*partMid.count()+sizeof(int);
+	uint partMidSize = 2*szInt*partMid.count()+szInt;
 	for (partMidIt = partMid.begin(); partMidIt != partMid.end(); ++partMidIt) {
 		
 		
@@ -379,8 +497,8 @@ uint BinHeader::getRecordSize( ) {
 	
 			
 			
-    return 11*sizeof(uint)+subj.length()+from.length()+date.length()+serverPart.count()*(2*sizeof(uint))+partNumSize \
-			+ 2*(sizeof(uint)*partSize.count()) + partMidSize + 2*sizeof(uint)*serverLowest.count();
+    return 12*szInt+subj.length()+from.length()+date.length()+serverPart.count()*(2*szInt)+partNumSize \
+			+ 2*(szInt*partSize.count()) + partMidSize + 2*szInt*serverLowest.count();
 }
 
 bool BinHeader::addPart(int part, Header *h) {
