@@ -203,6 +203,7 @@ void GroupList::slotAddGroup( Group* g)
 
 void GroupList::saveGroup( NewsGroup * ng )
 {
+	qDebug("Saving group");
 	char datamem[DATAMEM_SIZE];
 	char keymem[KEYMEM_SIZE];
 	
@@ -222,7 +223,7 @@ void GroupList::saveGroup( NewsGroup * ng )
 	
 	data.set_size(ng->getRecordSize());
 	
-	delete p;
+	delete [] p;
 	const char *i= (const char *) ng->getName();
 	memcpy(keymem, i, ng->getName().length());
 	key.set_size(ng->getName().length());
@@ -235,6 +236,9 @@ void GroupList::saveGroup( NewsGroup * ng )
 	} 
 	
 	groupDb->sync(0);
+// 	ng->listItem->setText(Unread_Col, QString::number(ng->unreadArticles));
+// 	ng->listItem->setText(Total_Col, QString::number(ng->totalArticles));
+	
 }
 
 void GroupList::slotExecuted( QListViewItem * item )
@@ -384,8 +388,9 @@ void GroupList::slotZeroSelected( )
 			
 		ng->totalArticles=0;
 		ng->unreadArticles=0;
-		ng->listItem->setText(Unread_Col, QString::number(ng->totalArticles));
-		ng->listItem->setText(Total_Col, QString::number(ng->unreadArticles));
+		
+// 		ng->listItem->setText(Unread_Col, QString::number(ng->totalArticles));
+// 		ng->listItem->setText(Total_Col, QString::number(ng->unreadArticles));
 		saveGroup(ng);
 		delete count;
 		++it;
@@ -444,6 +449,7 @@ void GroupList::slotDeleteAllArticles( int serverId)
 		//Don't need to expire if the server doesn't have the group
 		if (ng->high[serverId] != -1)
 			deleteAllArticles( ng, serverId);
+		qDebug("all articles deleted");
 		ng->low.remove(serverId);
 		ng->high.remove(serverId);
 		ng->stopUpdating();
@@ -463,7 +469,7 @@ void GroupList::slotDeleteAllArticles( int serverId)
 		
 		++it;
 	}
-// 	qDebug("GroupList::slotDeleteAllArticles() done");
+	qDebug("GroupList::slotDeleteAllArticles() done");
 	
 }
 
@@ -472,7 +478,7 @@ void GroupList::deleteAllArticles( NewsGroup *ng, int serverId )
 	Dbc *cursor;
 	
 	
-
+	qDebug("Reset server in group!");
 	if ((ng->getDb()->cursor(0, &cursor, DB_WRITECURSOR))!= 0) {
 		qDebug("Error creating cursor!!");
 		return;
@@ -502,7 +508,7 @@ void GroupList::deleteAllArticles( NewsGroup *ng, int serverId )
 			case BinHeader::Delete_Read:
 				if ((ret=cursor->del(0))!=0)
 					qDebug("Error deleting from db: %d", ret);
-				else ng->totalArticles--;
+				else ng->decTotal();
 				break;
 			case BinHeader::No_Delete:
 				p=bh->data();
@@ -510,7 +516,7 @@ void GroupList::deleteAllArticles( NewsGroup *ng, int serverId )
 				data.set_data(p);
 				data.set_size(bh->getRecordSize());
 				
-				if ((ret=cursor->put(NULL, &data, DB_CURRENT))!=0)
+				if ((ret=cursor->put(&key, &data, DB_CURRENT))!=0)
 					qDebug("Error updating post: %d", ret);
 				// 			else qDebug("Error! %d", ret);
 				delete p;
@@ -528,6 +534,9 @@ void GroupList::deleteAllArticles( NewsGroup *ng, int serverId )
 	
 
 	cursor->close();
+	ng->high[serverId]=0;
+	ng->low[serverId]=0;
+	saveGroup(ng);
 
 	
     qDebug("Elements in DB: %d", count);
@@ -606,6 +615,7 @@ void GroupList::slotNewGroup( QStringList entries, Group *g )
 	NewsGroup *tempNg = new NewsGroup(dbenv, entries[0], entries[1]);
 	tempNg->setAlias(entries[2]);
 	tempNg->setCategory(entries[3]);
+	tempNg->setDeleteOlder(entries[4].toInt());
 	groups.insert(tempNg->getName(), tempNg);
 	Servers::iterator it;
 	for (it =servers->begin(); it != servers->end(); ++it) {
@@ -660,6 +670,7 @@ void GroupList::slotModifyServerProperties( QStringList entries)
 					
 		ng->setCategory(entries[3]);	
 	}
+	ng->setDeleteOlder(entries[4].toInt());
 	
 	
 	saveGroup(ng);
@@ -741,6 +752,8 @@ void GroupList::slotCompactDbs( )
 	
 	while(it.current()) {
 		ng=it.current();
+		kdDebug() << "Compacting " << ng->getName() << endl;
+		qDebug("Compacting %s", ng->getName().latin1());
 		++it;
 		gCount++;
 		//-Create the new Db.
@@ -762,13 +775,22 @@ void GroupList::slotCompactDbs( )
 		previous=QTime::currentTime();
 		BinHeader *bh;
 		while (cursor->get(&key, &data, DB_NEXT) != DB_NOTFOUND  ) {
-			totalArticles++;
-			bh = new BinHeader((uchar*)data.get_data());
-			if (bh->getStatus() == BinHeader::bh_new)
-				unreadArticles++;
-			delete bh;
-			if (newDb->put(0, &key, &data, 0) != 0)
-				qDebug("Error putting data into new db!");
+			
+			int size = data.get_size();
+			int storedSize;
+			uchar *p = (uchar*) data.get_data();
+			memcpy(&storedSize, p + size-szInt, szInt);
+			if (storedSize != size) {
+				kdDebug() << "Error inside db: stored size different than actual key size!\n";
+			} else {
+				totalArticles++;
+				bh = new BinHeader((uchar*)data.get_data());
+				if (bh->getStatus() == BinHeader::bh_new)
+					unreadArticles++;
+				delete bh;
+				if (newDb->put(0, &key, &data, 0) != 0)
+					qDebug("Error putting data into new db!");
+			}
 			
 			free(key.get_data());
 			memset(&key, 0, sizeof(key));
@@ -787,6 +809,8 @@ void GroupList::slotCompactDbs( )
 				previous=QTime::currentTime();
 			}
 		}
+		kdDebug() << ng->getName() << " Compacted\n";
+		qDebug("%s Compacted", ng->getName().latin1());
 		cursor->close();
 		
 		ng->setTotal(totalArticles);
@@ -832,6 +856,9 @@ void GroupList::slotSaveSettings( NewsGroup *ng, bool unread, bool complete)
 // 	qDebug("Saving group");
 	ng->showOnlyUnread=unread;
 	ng->showOnlyComplete=complete;
+	ng->listItem->setText(Unread_Col, QString::number(ng->unreadArticles));
+	ng->listItem->setText(Total_Col, QString::number(ng->totalArticles));
+// 	qDebug("Saving settings");
 // 	qDebug("Unread: %d", ng->onlyUnread());
 // 	qDebug("Complete: %d", ng->onlyComplete());
 	saveGroup(ng);
@@ -841,8 +868,7 @@ void GroupList::updateCurrent( NewsGroup * ng )
 {
 	groups[ng->getName()]->listItem->setEnabled(false);
 	
-	//TODO: disable action? What if the user starts clicking the update button
-	//like a mad monkey?
+	///@todo disable action? What if the user starts clicking the update button like a mad monkey?
 	
 	emit updateNewsGroup(ng,0 );
 	
