@@ -26,7 +26,7 @@
 #include "progress.h"
 #include <errno.h>
 #include <kprogress.h>
-
+#include <qregexp.h>
 
 
 NewHeaderList::NewHeaderList(NewsGroup *_ng, Servers *_servers, QString caption, QWidget *parent): KMdiChildView (caption, parent), ng(_ng), servers(_servers) 
@@ -45,12 +45,20 @@ NewHeaderList::NewHeaderList(NewsGroup *_ng, Servers *_servers, QString caption,
 	m_clearButton->setIconSet(KGlobal::iconLoader()->loadIcon("locationbar_erase", KIcon::Small, 0, false));
 	QLabel *label=new QLabel(this);
 	label->setText("Filter by subject:");
+	m_enterButton=new QToolButton(this);
+	m_enterButton->setIconSet(KGlobal::iconLoader()->loadIcon("key_enter", KIcon::Small, 0, false));
+	
+	regExpCheck = new QCheckBox("Regular expression", this);
+	
 	
 	filterLayout->addWidget(m_clearButton);
 	filterLayout->addWidget(label);
+	
 	filterLayout->setSpacing(4);
 	m_filterEdit=new KLineEdit(this);
 	filterLayout->addWidget(m_filterEdit);
+	filterLayout->addWidget(regExpCheck);
+	filterLayout->addWidget(m_enterButton);
 	
 	
 	m_headerList=new KListView(this);
@@ -79,7 +87,9 @@ NewHeaderList::NewHeaderList(NewsGroup *_ng, Servers *_servers, QString caption,
 	setIcon(KGlobal::iconLoader()->loadIcon("icon_newsgroup",KIcon::Small, KIcon::SizeSmall, false));
 	loadGroup(ng->getDb() );
 // 	m_loadGroup(ng->getDb());
-	connect(m_filterEdit, SIGNAL(returnPressed(const QString &)), this, SLOT(slotActivateFilter(const QString& )));
+// 	connect(m_filterEdit, SIGNAL(returnPressed(const QString &)), this, SLOT(slotActivateFilter(const QString& )));
+	connect(m_filterEdit, SIGNAL(returnPressed()), this, SLOT(slotEnterButtonClicked()));
+	connect(m_enterButton, SIGNAL(clicked()), this, SLOT(slotEnterButtonClicked()));
 	connect(m_headerList, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)), this, SLOT(slotContextMenu(QListViewItem*, const QPoint& )));
 	connect(m_clearButton, SIGNAL(clicked()), this, SLOT(slotClearButtonClicked()));
 	//Check if we override global KDE setting
@@ -91,7 +101,7 @@ NewHeaderList::NewHeaderList(NewsGroup *_ng, Servers *_servers, QString caption,
 	showOnlyComplete=ng->onlyComplete();
 	if (showOnlyComplete || showOnlyNew)
 		filter(m_filterEdit->text());
-	
+	m_headerList->setFocus();
 	
 }
 
@@ -104,6 +114,8 @@ void NewHeaderList::loadGroup( Db * db )
     m_headerList->setSorting(-1);
     m_headerList->clear();
 	hiddenItems.clear();
+
+
 	if (allowExpansion)
 		m_headerList->setRootIsDecorated(TRUE);
 	else m_headerList->setRootIsDecorated(false);
@@ -126,9 +138,10 @@ void NewHeaderList::loadGroup( Db * db )
     bindata->set_data(bindatamem);
 	
 	Dbc *cursor;
-	BinHeader *bh;
+// 	BinHeader *bh;
+	SmallBinHeader *sbh;
 	
-	db->cursor(NULL, &cursor, 0);
+	db->cursor(NULL, &cursor, DB_WRITECURSOR);
 	int count=0;
 	
 // 	Progress *pd= new Progress(ng->getName(), this);
@@ -141,8 +154,10 @@ void NewHeaderList::loadGroup( Db * db )
 	kpd->show();
 	
 	QTime current, previous;
+	QDateTime hDate;
+	QDateTime cDate = QDateTime::currentDateTime();
 	previous=QTime::currentTime();
-	
+	int deleteOlder = ng->getDeleteOlder();
 	
 	
 	while ((ret=cursor->get(binkey, bindata, DB_NEXT)) != DB_NOTFOUND) {
@@ -164,12 +179,52 @@ void NewHeaderList::loadGroup( Db * db )
 			
 		}
 		if (ret==0) {
+			/*
 			bh=new BinHeader((uchar*)bindatamem);
 // 			new BHListViewItem(m_headerList, bh, db, servers);
 			
 			new BHListViewItem(m_headerList, bh, db, showFrom, showDate,  servers);
 			count++;
 			delete bh;
+			
+			*/
+			int size = bindata->get_size();
+			int storedSize;
+			memcpy(&storedSize, bindatamem+size-szInt, szInt);
+			if (storedSize != size) {
+				qDebug("Error! Corruptd db. Storedsize: %d; size: %d", storedSize, size);
+				//delete the header from the db!
+				
+			} else {
+				sbh=getSmallBinHeader(bindatamem);
+				//Check deleteOlder
+				if (deleteOlder > 0) {
+					//check and, if needed, delete the header
+					hDate = QDateTime::fromString(sbh->date);
+					if (hDate.daysTo(cDate) > deleteOlder ) {
+// 						qDebug("Current date: %s", (const char *) cDate.toString());
+						qDebug("Header date: %s", (const char *) hDate.toString());
+						if (sbh->status == BinHeader::bh_new)
+							ng->decUnread();
+						//dec. total articles..
+						ng->decTotal();
+						cursor->del(0);
+						
+						
+					} else {
+						new BHListViewItem(m_headerList, sbh, db, showFrom, showDate, servers);
+						count++;
+						
+					}
+					delete sbh;
+					
+				} else {
+					new BHListViewItem(m_headerList, sbh, db, showFrom, showDate, servers);
+					count++;
+					delete sbh;
+				}
+			}
+			
 			current=QTime::currentTime();
 			if (previous.secsTo(current) > 1)  {
 // 				pd->setProgress( uint((float(count)/float(ng->getTotal())) * 100));
@@ -188,8 +243,7 @@ void NewHeaderList::loadGroup( Db * db )
 	
 // 	qDebug("Count: %d", count);
 	cursor->close();
-
-	
+	emit sigSaveSettings(ng, showOnlyNew, showOnlyComplete);
 	m_headerList->setUpdatesEnabled(TRUE);
 	m_headerList->setSorting(0);
 	
@@ -204,14 +258,6 @@ void NewHeaderList::loadGroup( Db * db )
 
 
 
-void NewHeaderList::slotActivateFilter( const QString &s )
-{
-	
-	filter(s);
-	
-
-	
-}
 
 void NewHeaderList::slotContextMenu( QListViewItem *, const QPoint & p )
 {
@@ -257,8 +303,7 @@ void NewHeaderList::slotDownloadSelected(bool first, bool view, QString dir)
 		if (first)
 			--it;
 		else ++it;
-		if (selected->depth() == 0) {
-			
+		if (selected->depth() == 0 ) {
 			int ret;
 			count++;
 			QString index = selected->getSubject() +selected->getFrom();
@@ -278,7 +323,7 @@ void NewHeaderList::slotDownloadSelected(bool first, bool view, QString dir)
 				datamem=new uchar[data->get_size()+1000];
 				data->set_ulen(data->get_size()+1000);
 				data->set_data(datamem);
-				delete p;
+				delete [] p;
 				qDebug("Exiting growing array cycle");
 				ret=ng->getDb()->get(0,key, data, 0);
 				
@@ -299,7 +344,7 @@ void NewHeaderList::slotDownloadSelected(bool first, bool view, QString dir)
 				selected->repaint();
 				uchar *p=bh->data();
 				memcpy(datamem, p, bh->getRecordSize());
-				delete p;
+				delete [] p;
 				data->set_size(bh->getRecordSize());
 				ret=ng->getDb()->put(NULL, key, data, 0);
 				if (ret!=0)
@@ -317,7 +362,7 @@ void NewHeaderList::slotDownloadSelected(bool first, bool view, QString dir)
 				
 // 				delete bh;
 					
-			}else qDebug("Error retrieving record: %d", ret);
+			}else  kdDebug() << "Error retrieving record: " << ret << endl;
 		}
 		
 		
@@ -361,6 +406,73 @@ bool NewHeaderList::prepareToClose( bool confirmation)
 	
 }
 
+void NewHeaderList::slotDelSelected()
+{
+	qDebug("Delselected");
+	QPtrList<QListViewItem> selection=m_headerList->selectedItems();
+	if (selection.isEmpty())
+		return;
+	
+	
+	Dbt key;
+	memset(&key, 0, sizeof(key));
+	
+	
+	QPtrListIterator<QListViewItem> it(selection);
+	BHListViewItem *selected;
+
+	uint count=selection.count();
+	uint height=0;
+	
+	selected=(BHListViewItem*) selection.getFirst();
+	QRect rect;
+	rect=m_headerList->itemRect(selected);
+	//Check if the first item of the selection is outside the viewport. If yes, 
+	//we have to scroll the listview...
+// 	QListViewItem *first=0;
+	if (rect.y() == 0) {
+		height=it.current()->height();
+// 		first = selection.getLast()->itemBelow();
+	}
+	while ((selected=(BHListViewItem*)it.current())) {
+		
+		++it;
+		if (selected->depth() == 0) {
+			
+			int ret;
+			QString index = selected->getSubject()+selected->getFrom();
+			const char *k= (const char *)index;
+			key.set_data((void*)k);
+			key.set_size(index.length());
+						
+			ret=ng->getDb()->del(NULL, &key,0);
+			if (ret==0) {
+				ng->decTotal();
+				if (selected->getStatus() == BinHeader::bh_new)
+					ng->decUnread();
+				selected->setSelected(false);
+				m_headerList->removeItem(selected);
+				
+					
+			}else qDebug("Error retrieving record: %d", ret);
+		
+			
+			
+			memset(&key, 0, sizeof(key));
+		
+		}
+		
+		
+	}
+// 	if (first)
+// 		m_headerList->ensureItemVisible(first);	
+	m_headerList->scrollBy(0, -(count*height));
+	m_headerList->adjustColumn(0);
+	emit updateFinished(ng);
+	
+}
+
+
 void NewHeaderList::markSelectedAs( int what)
 {
 	
@@ -502,6 +614,48 @@ void NewHeaderList::slotShowOnlyNew( )
 	filter(m_filterEdit->text());
 }
 
+void NewHeaderList::filter( const QRegExp & rx )
+{
+	m_filterEdit->setEnabled(false);
+	m_headerList->setEnabled(false);
+	
+	
+	//for now we check each item against 3 filters: subj, complete, new
+	
+	BHListViewItem *item=(BHListViewItem*)m_headerList->firstChild();
+	while (item) {
+		
+		if ((item->text(Subj_Col).find(rx, 0) == -1) ||
+				   ( showOnlyComplete && (!item->isComplete())) ||
+				   (showOnlyNew && (!item->isNew()))) 
+		{
+			hiddenItems.insert(item->getSubject() + item->getFrom() , item);
+			BHListViewItem *temp=(BHListViewItem*)item->nextSibling();
+			m_headerList->takeItem(item);
+			item=temp;
+			
+		} else item=(BHListViewItem*)item->nextSibling();
+	}
+	QDictIterator<BHListViewItem> it(hiddenItems);
+	while (it.current()) {
+// 		if ((it.currentKey().find(s, 0, false) != -1) &&
+		if ((it.current()->getSubject().find(rx,0) != -1) &&
+				   (!showOnlyComplete || it.current()->isComplete()) &&
+				   (!showOnlyNew || it.current()->isNew()))
+			  
+		{ //item found!
+			m_headerList->insertItem(it.current());
+			hiddenItems.remove(it.currentKey());
+		} else ++it;
+	}
+		
+	m_headerList->setEnabled(true);
+	m_filterEdit->setEnabled(true);
+	
+}
+
+
+
 void NewHeaderList::filter( const QString &s)
 {
 	m_filterEdit->setEnabled(false);
@@ -524,45 +678,28 @@ void NewHeaderList::filter( const QString &s)
 			m_headerList->takeItem(item);
 			item=temp;
 			
-			
 		} else item=(BHListViewItem*)item->nextSibling();
 		
-// 		else if ( showOnlyComplete && (!item->isComplete())) {
-// 			Hide item
-// 			
-// 		} else if (showOnlyNew && (!item->isNew())) {
-// 			Hide item
-// 		} else item=(BHListViewItem*)item->nextSibling();
 	}
-	//Now scan the "hidden" list...
 	QDictIterator<BHListViewItem> it(hiddenItems);
 	while (it.current()) {
 // 		if ((it.currentKey().find(s, 0, false) != -1) &&
 		if ((it.current()->getSubject().find(s, 0, false) != -1) &&
-				(!showOnlyComplete || it.current()->isComplete()) &&
-				(!showOnlyNew || it.current()->isNew()))
+				   (!showOnlyComplete || it.current()->isComplete()) &&
+				   (!showOnlyNew || it.current()->isNew()))
 			  
 		{ //item found!
 			m_headerList->insertItem(it.current());
 			hiddenItems.remove(it.currentKey());
 		} else ++it;
 	}
-	
-	
-	m_filterEdit->setEnabled(true);
+		
 	m_headerList->setEnabled(true);
+	m_filterEdit->setEnabled(true);
 	m_headerList->adjustColumn(0);
 	
 }
 
-BHListViewItem  *NewHeaderList::hideItem( BHListViewItem *item )
-{
-	hiddenItems.insert(item->text(Subj_Col), item);
-	BHListViewItem *temp=(BHListViewItem*)item->nextSibling();
-	m_headerList->takeItem(item);
-	return temp;
-	
-}
 
 void NewHeaderList::reload( )
 {
@@ -666,6 +803,23 @@ void NewHeaderList::slotViewArticle( )
 	slotDownloadSelected(true, true);
 }
 
+void NewHeaderList::closeAndMark( )
+{
+	QCloseEvent *e = new QCloseEvent();
+	ng->setView(NULL);
+	markGroupAsRead();
+	KMdiChildView::closeEvent(e);
+	
+}
+
+void NewHeaderList::closeAndNoMark( )
+{
+	QCloseEvent *e = new QCloseEvent();
+	ng->setView(NULL);
+	KMdiChildView::closeEvent(e);
+	
+}
+
 void NewHeaderList::closeEvent( QCloseEvent * e )
 {
 	
@@ -824,6 +978,7 @@ void NewHeaderList::markGroupAsRead( )
 		}
 		++it;
 	}
+	
 // 	qDebug("Marked invisible items");
 	//Finally, emit the update signal
 	emit updateFinished(ng);
@@ -847,10 +1002,65 @@ void NewHeaderList::slotItemExecuted( )
 	slotViewArticle();
 }
 
+SmallBinHeader * NewHeaderList::getSmallBinHeader( char * p )
+{
+	char *i=&p[0];
+	int szInt=sizeof(int);
+	int count;
+	memcpy(&count, i, szInt);
+// 	kdDebug() << "Count: " << count << endl;
+	//Skip serverLowest
+	i+=(count*2*szInt+szInt);
+	SmallBinHeader *sbh = new SmallBinHeader;
+	i=retrieve(i, sbh->subj);
+// 	kdDebug() << "Subj: "<< sbh->subj << endl;
+	i=retrieve(i, sbh->from);
+	i=retrieve(i, sbh->date);
+	
+	memcpy(&(sbh->parts), i, szInt);
+	i+=szInt;
+	
+	memcpy(&(sbh->missingParts), i, szInt);
+	i+=szInt;
+	
+	memcpy(&(sbh->status), i, szInt);
+	//Skip lines
+	i+=2*szInt;
+	memcpy(&(sbh->size), i, szInt);
+	i+=szInt;
+	
+	int id, parts;
+	memcpy(&count, i, szInt);
+	i+=szInt;
+	for (int j=0; j < count; j++) {
+		memcpy(&id, i, szInt);
+		i+=szInt;
+		memcpy(&parts, i, szInt);
+		i+=szInt;
+		sbh->serverParts.insert(id, parts);
+		
+	}
+	return sbh;
+	
+	
+}
 
-
-
-
+void NewHeaderList::slotEnterButtonClicked( )
+{	
+	
+	if (regExpCheck->isChecked()) {
+		QRegExp rx;
+		rx.setPattern(m_filterEdit->text());
+		rx.setCaseSensitive(false);
+		if (rx.isValid())
+			filter(rx);
+		else KMessageBox::error(this, i18n("Invalid regular expression!"),i18n("Error") );
+				
+	} else {
+		filter(m_filterEdit->text());
+	}
+	
+}
 
 /*$SPECIALIZATION$*/
 
