@@ -33,6 +33,7 @@
 #include <assert.h>
 #include <kkeydialog.h>
 #include <kdeversion.h>
+#include <qfile.h>
 
 #include <kdeversion.h>
 
@@ -49,32 +50,37 @@ klibido::klibido()
     // set the shell's ui resource file
     setXMLFile("klibidoui.rc");
 
-	setCaption("KLibido 0.2.3");
+	setCaption("KLibido 0.2.4.1");
 	dbenv=0;
 	fViewer=0;
 	qMgr=0;
-	if (!Config().configured) {
-		slotSettingsConfigure();
-	} else {
-		//Check and create the directories
-		checkAndCreateDir(Config().dbDir);
-		checkAndCreateDir(Config().decodeDir);
-		checkAndCreateDir(Config().tmpDir);
-		setupDbEnv(Config().dbDir);
-		
-		migrateDbs();
-	}
-	
-	
-	
+
+
+  if (!Config().configured || !dirCreated()) {
+    if(!slotSettingsConfigure()) {
+		KMessageBox::error(this, i18n("Configuration aborted. KLibido will now exit"), i18n("Configuration error")); 	
+      	exit(-1);
+    }
+  }
+  setupDbEnv(Config().dbDir);
+  if (!dbsExists(Config().dbDir)) {
+    int result=KMessageBox::warningYesNo(this, i18n("Can't find all the files of your database in %1. Do you want to recreate it ?").arg(Config().dbDir), i18n("question"));
+    switch (result) {
+      case KMessageBox::Yes:
+        createDbs();
+        break;
+      case KMessageBox::No:
+        exit(-1);
+        break;
+
+    }
+  }
+  else  migrateDbs();
 	
 	
 	setAutoSaveSettings();
 // 	setupServers();
 	
-	//Now, check the Db versions...
-	
-	
 	
 	accel = new KAccel(this);
 	groupList=new GroupList("newsgroups", dbenv,  &servers, this);
@@ -93,7 +99,8 @@ klibido::klibido()
 	connect(serversList, SIGNAL(serverSelected(bool )), this, SLOT(slotServerSelection(bool )));
 	connect(serversList, SIGNAL(sigHaveServers(bool )), this, SLOT(slotHaveServers(bool)));
 	
-	//Test!
+	//tabWidget = tabWidget();
+	connect(tabWidget(), SIGNAL(contextMenu(QWidget*, const QPoint& )), this, SLOT(slotPopupTabMenu(QWidget*, const QPoint &)));
 	
 	
 	
@@ -101,7 +108,12 @@ klibido::klibido()
 	groupDb=groupList->getGroupDb();
 	qMgr=new QMgr(&servers, groupDb, "Q Manager", this);
 	qMgr->setDbEnv(Config().dbDir, dbenv, groupList);
+
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)
 	addWindow(qMgr, KMdi::StandardAdd, 0);
+#else
+	addWindow(qMgr, KMdi::StandardAdd);
+#endif
 	qMgr->setDecodeOverwrite(Config().overwriteExisting);
 	
 	aGroups = new AvailableGroups(dbenv, &servers, this);
@@ -135,6 +147,7 @@ klibido::klibido()
 	connect(qMgr, SIGNAL(sigItemsSelected(bool )), this, SLOT(slotQueueItemsSelected(bool )));
 	connect(qMgr, SIGNAL(sigQPaused(bool )), this,SLOT(slotQPaused(bool )));
 	connect(serversList, SIGNAL(sigServerAdded(NntpHost* )), qMgr, SLOT(slotAddServer(NntpHost* )));
+	connect(serversList, SIGNAL(sigServerAdded(NntpHost*)), this, SLOT(slotAddServerWidget(NntpHost*)));
 	connect(this, SIGNAL(getGroups(AvailableGroups* )), qMgr, SLOT(slotAddListItem(AvailableGroups* )));
 // 	addWindow(qMgr);
 	connect(serversList, SIGNAL(sigAddThread(int, int )), qMgr, SLOT(slotAddThread(int, int )));
@@ -145,7 +158,9 @@ klibido::klibido()
 	connect(this, SIGNAL(sigOkToDeleteServer()), serversList, SLOT(slotDeleteServer()));
 	connect(serversList, SIGNAL(deleteServer(int )), groupList, SLOT(slotDeleteAllArticles(int )));
 	connect(serversList, SIGNAL(deleteServer(int )), aGroups, SLOT(slotDeleteServer(int )));
+	connect(serversList, SIGNAL(deleteServer(int)), this, SLOT(slotDeleteServerWidget(int)));
 	connect(serversList, SIGNAL(deleteServer(int)), qMgr, SLOT(slotDeleteServerQueue(int )));
+	connect(serversList, SIGNAL(sigServerSpeed(int, int )), this, SLOT(slotUpdateSpeed(int, int )));
 	connect(qMgr, SIGNAL(viewItem(KURL )), this, SLOT(slotViewFile(KURL )));
 	connect(qMgr, SIGNAL(sigThreadPaused(int, int, int )), serversList, SLOT(slotThreadPaused(int, int, int )));
 	connect(serversList, SIGNAL(sigPauseThread(int, int )), qMgr, SLOT(slotPauseThread(int, int )));
@@ -153,11 +168,21 @@ klibido::klibido()
 	
 	statusBar()->show();
 	statusBar()->message(i18n("Ready"));
+	
+	
 	if ( Config().singleViewTab && !fViewer) {
 		fViewer=new FileViewer(KURL::KURL(QString::null), this);
 		addWindow(fViewer);
 		activateView(qMgr);
 	}
+	
+	//servers widget
+	Servers::iterator it;
+	for (it = servers.begin() ; it != servers.end(); ++it) {
+		sWidgets[it.key()] = new QStatusServerWidget(it.data()->name, this);
+		statusBar()->addWidget(sWidgets[it.key()], 0, true);
+		
+	}
 
 	//Statusbar widget
 	qTWidget= new QToolbarWidget(this);
@@ -165,10 +190,10 @@ klibido::klibido()
 	connect(qMgr, SIGNAL(queueInfo(unsigned long long, unsigned long long)), qTWidget, SLOT(slotQueueInfo(unsigned long long,  unsigned long long)));
 // 	statusBar()->addWidget(*tIcon, 0, true);
 	
-	/*
-	if (!isFakingSDIApplication()) {
+	
+/*	if (!isFakingSDIApplication()) {
     menuBar()->insertItem( tr("&Window"), windowMenu());
- 	}*/
+	}*/
 	hideViewTaskBar();
 	createGUI(0);
 	
@@ -199,8 +224,13 @@ klibido::klibido()
 #if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)	
 	tabWidget()->setTabCloseActivatePrevious(true);
 #endif
-	
-
+	wMenu = (KPopupMenu*) factory()->container("window", this);
+// 	wMenu = new KPopupMenu(0, "window");
+// 	menuBar()->insertItem("&Window", -1, 5);
+// 	menuBar()->setItemEnabled(10000, true);
+	/// @todo manage window menu for fViewer
+	connect(wMenu, SIGNAL(activated(int)), this, SLOT(slotWMenuActivated(int)));
+	connect(wMenu, SIGNAL(aboutToShow()), this, SLOT(slotAboutToShow()));
 	qMgr->checkQueue();
 	
 }
@@ -209,7 +239,7 @@ klibido::~klibido()
 {
 	qDebug("KLibido::~klibido()");
 	if (qMgr)
-		KMdiMainFrm::closeWindow(qMgr);
+		closeWindow(qMgr);
 // 		
 // 	if (fViewer)
 // 		KMdiMainFrm::closeWindow(fViewer);
@@ -221,6 +251,7 @@ klibido::~klibido()
 	delete serversList;
 	delete aGroups;
 	dbenv->close(0);
+// 	delete dbenv;
 	
 }
 
@@ -236,6 +267,14 @@ void klibido::setupDbEnv(QString dir )
 {
 	dbenv=new DbEnv(DB_CXX_NO_EXCEPTIONS);
 	dbenv->set_cachesize(0, 4000000, 0);
+/*
+  struct stat statBuffer;
+  if (stat(dir, &statBuffer) != 0) {
+    KMessageBox::error(this, i18n("Can open database directory : %1").arg(dir), i18n("Error!"));
+    exit(-1);
+  }*/
+ // printf("%d\n",stat(dir, &statBuffer));
+  
 // 	dbenv->set_flags(DB_CDB_ALLDB,1);
 //     if ((ret=dbenv->open((const char *) dir, DB_CREATE | DB_THREAD | DB_INIT_CDB |DB_INIT_MPOOL | DB_PRIVATE, 0644))!= 0)
 	const char *d=dir.latin1();
@@ -291,11 +330,12 @@ void klibido::setupActions( )
 	dSelected=new KAction(i18n("Download selected"), CTRL + Key_D , 0, 0,actionCollection(), "download_selected");
 	dSelectedFirst=new KAction(i18n("Download selected first"), 0, 0, 0,actionCollection(), "download_selected_first");
 	dSelectedTo=new KAction(i18n("Download to dir..."), 0, 0, 0, actionCollection(), "download_to_dir");
+	
 	showOnlyComplete=new KToggleAction(i18n("Show only complete articles"), Key_C, 0, 0,actionCollection(), "show_only_complete");
 	showOnlyNew=new KToggleAction(i18n("Show only unread articles"), Key_U, 0, 0,actionCollection(), "show_only_new");
 	markAsRead=new KAction(i18n("Mark as read"), 0,0,actionCollection(), "mark_as_read");
 	markAsUnread=new KAction(i18n("Mark as unread"), 0, 0, actionCollection(), "mark_as_unread");
-	
+	delHeader = new KAction(i18n("Delete"), 0, 0, actionCollection(), "delete_header");
 	
 	//Q actions
 	
@@ -306,10 +346,16 @@ void klibido::setupActions( )
 	(void)new KAction(i18n("Resume"), 0, qMgr, SLOT(slotResumeSelected()), actionCollection(), "resume_selected");
 	(void)new KAction(i18n("Cancel"), 0, qMgr, SLOT(slotCancelSelected()), actionCollection(), "cancel_selected");
 	(void)new KAction(i18n("Move to top"), 0, qMgr, SLOT(slotMoveSelectedToTop()), actionCollection(), "move_to_top");
-	(void)new KAction(i18n("Close active view"), CTRL + Key_W, this, SLOT(slotCloseActiveView()), actionCollection(), "close_active_view");
+// 	(void)new KAction(i18n("Close active view"), CTRL + Key_W, this, SLOT(()), actionCollection(), "close_active_view");
 	
 // 	accel->insert("close_view", i18n("Close Active view"), i18n("Closes the active view"), CTRL + Key_W, this, 
 // 			SLOT(slotCloseActiveView()));
+	closeAction = new KAction(i18n("Close window"), CTRL + Key_W, this, SLOT(slotCloseWindow()), actionCollection(), "close_window");
+// 	(void)new KAction(i18n("Close and mark as read"), 0, this, SLOT(slotCloseAndMark()), actionCollection(), "close_mark");
+// 	(void)new KAction(i18n("Close and don't mark as read"), 0, this, SLOT(slotCloseAndNoMark()), actionCollection(), "close_nomark");
+	
+// 	assert(wMenu);
+// 	wMenu->insertItem("Queue manager");
 	
 }
 
@@ -339,6 +385,7 @@ void klibido::slotOpenNewsGroup( NewsGroup * ng )
 	
 	NewHeaderList * view=new NewHeaderList(ng, &servers, ng->getName(), this);
 	ng->setView( view );
+	groupList->saveGroup(ng);
 	connect(view, SIGNAL(articlePopup(const QPoint &)), this, SLOT(slotPopupArticleMenu(const QPoint& )));
 	
 	connect(view, SIGNAL(activated(KMdiChildView*)), this, SLOT(slotGroupActivated(KMdiChildView* )));
@@ -355,9 +402,13 @@ void klibido::slotOpenNewsGroup( NewsGroup * ng )
 	
 	addWindow(view);
 	activateView(view);
+// 	int index = tabWidget()->indexOf(view);
+// 	wMenu->insertItem(QString::number(index) + ' ' + view->caption(), index, -1);
+// 	wMenu->setItemParameter(index, 99);
 	
 	stateChanged("no_article_window", StateReverse);
 	
+	
 
 	
 }
@@ -397,6 +448,11 @@ void klibido::slotPopupCatMenu( const QPoint &p )
 	pop->popup(p);
 }
 
+void klibido::slotPopupWindowMenu(const QPoint &p, KMdiChildView *v)
+{
+	WPopupMenu *pop = new WPopupMenu(this, v->caption() , v);
+	pop->popup(p);
+}
 
 void klibido::slotPopupArticleMenu( const QPoint &p )
 {
@@ -452,16 +508,19 @@ void klibido::slotGroupActivated( KMdiChildView * view)
 	disconnect(dSelected, SIGNAL(activated()), 0, 0);
 	disconnect(markAsRead, SIGNAL(activated()),0,0);
 	disconnect(markAsUnread, SIGNAL(activated()), 0,0);
+	disconnect(delHeader, SIGNAL(activated()), 0, 0);
 	disconnect(showOnlyNew, SIGNAL(activated()), 0,0);
 	disconnect(showOnlyComplete, SIGNAL(activated()), 0, 0);
 	
 	
+	
 	connect(viewArticle, SIGNAL(activated()), view, SLOT(slotViewArticle()));
 	connect(dSelected, SIGNAL(activated()), view, SLOT(slotDownloadSelected()));
 	connect(dSelectedFirst, SIGNAL(activated()), view, SLOT(slotDownloadSelectedFirst()));
 	connect(dSelectedTo, SIGNAL(activated()), view, SLOT(slotDownloadToDir()));
 	connect(markAsRead, SIGNAL(activated()), view, SLOT(slotMarkSelectedAsRead()));
 	connect(markAsUnread, SIGNAL(activated()), view, SLOT(slotMarkSelectedAsUnread()));
+	connect(delHeader, SIGNAL(activated()), view, SLOT(slotDelSelected()));
 	connect(showOnlyNew, SIGNAL(activated()), view, SLOT(slotShowOnlyNew()));
 	connect(showOnlyComplete, SIGNAL(activated()), view, SLOT(slotShowOnlyComplete()));
 	
@@ -489,7 +548,7 @@ void klibido::slotShowQueue( )
 
 }
 
-void klibido::slotSettingsConfigure( )
+bool klibido::slotSettingsConfigure( )
 {
 	PrefDialog* pf=new PrefDialog(this, i18n("Preferences"));
 	QSize qs(530, 410);
@@ -505,7 +564,11 @@ void klibido::slotSettingsConfigure( )
 				//Didn't have the file viewer, create it...
 				fViewer=new FileViewer(KURL::KURL(QString::null), this);
 				KMdiChildView *prevActiveWindow=activeWindow();
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,3,0)				
 				addWindow(fViewer,KMdi::StandardAdd, 1);
+#else
+				addWindow(fViewer, KMdi::StandardAdd);
+#endif
 				if (prevActiveWindow)
 					activateView(prevActiveWindow);
 				else if (qMgr)
@@ -526,91 +589,25 @@ void klibido::slotSettingsConfigure( )
 		if (qMgr)
 			qMgr->setDecodeOverwrite(Config().overwriteExisting);
 		
-		//Create the version Db...
-		if (!dbenv)
-			setupDbEnv(Config().dbDir);
-		else kdDebug() << "slotSettingsConfigure(): Dbenv already opened\n";
-		Db *versionDb=new Db(dbenv, 0);
-		if (versionDb->open(NULL, "versiondb" , NULL, DB_BTREE, DB_CREATE | DB_THREAD , 0644) != 0)
-			qDebug("Error opening versiondb");
-	//Keys, data have fixed size:
-	//DATA is a uint
-	//KEY is one of GROUPDB, AVAILABLEGROUPSDB, HEADERDB
-		Dbt key,data;
-		
-		
-		uint groupDbVersion;
-		groupDbVersion=GROUPDB_VERSION;
-		data.set_data(&groupDbVersion);
-		data.set_size(sizeof(uint));
-			//,availableGroupsDbVersion, headersDbVersion;
-		int ret;
-		
-		//Put groupDb version in versionDb
-		const char *groupDbKey="GROUPDB";
-		key.set_data((char *)groupDbKey);
-		key.set_size(strlen(groupDbKey));
-		ret=versionDb->put(0, &key, &data, 0);
-		if (ret != 0) {
 
-			qDebug("Error puttting groupdb version: %s", dbenv->strerror(ret));
-		
-		}
-		
-		//put headerDb version in versionDb
-		memset(&key, 0, sizeof(key));
-// 		key.set_flags(DB_DBT_MALLOC);
-		memset(&data, 0, sizeof(data));
-// 		data.set_flags(DB_DBT_MALLOC);
-		//I could use groupDbVersion, but it's better to create a properly named local
-		//variable to avoid confusion...
-		uint headerDbVersion;
-		headerDbVersion=HEADERDB_VERSION;
-		data.set_data(&headerDbVersion);
-		data.set_size(sizeof(uint));
-	
-		const char *headerDbKey="HEADERDB";
-		key.set_data((char*)headerDbKey);
-		key.set_size(strlen(headerDbKey));
-		ret=versionDb->put(0, &key, &data, 0);
-		if (ret != 0) {
-			qDebug("Error putting headerdb version: %s", dbenv->strerror(ret));
-		}
-		
-		//serverdb key...
-		memset(&key, 0, sizeof(key));
-		memset(&data, 0, sizeof(data));
-		uint serverDbVersion = SERVERDB_VERSION;
-		data.set_data(&serverDbVersion);
-		data.set_size(sizeof(uint));
-		const char *serverDbKey="SERVERDB";
-		key.set_data((char*)serverDbKey);
-		key.set_size(strlen(serverDbKey));
-		ret=versionDb->put(0, &key, &data, 0);
-		if (ret != 0)
-			kdDebug() << "Error putting serverdb version: " << ret << endl;
-		
-		
-		
-		
-		
-		versionDb->close(0);
-		delete versionDb;
-		versionDb=0;
+    if (!dbsExists(Config().dbDir))
+      createDbs();
 		
 		
+	  return true;	
 	}
+    return false;
 }
 
 void klibido::closeWindow( KMdiChildView * pWnd, bool layoutTaskBar  )
 {
-	qDebug("Closing window");
-	if (pWnd == qMgr)
-		return;
-	if (pWnd == fViewer && Config().singleViewTab) {
+	qDebug("Closing window %s", (const char *) pWnd->caption());
+// 	if (pWnd == qMgr)
+// 		return;
+/*	if (pWnd == fViewer && Config().singleViewTab) {
 		
 		return;
-	}
+	}*/
 	
 	if (pWnd == aGroups->view()) {
 		aGroups->prepareToClose();
@@ -716,15 +713,16 @@ void klibido::slotGetGroups( )
 
 void klibido::slotShowGroups( )
 {
+	Q_ASSERT(aGroups);
 	if (!aGroups->view())
 	{
 		enable(false);
 		aGroups->loadGroups();
 		addWindow(aGroups->view());
 		enable(true);
-		} else {
+	} else {
 			activateView(aGroups->view());
-		}
+	}
 
 }
 
@@ -814,7 +812,9 @@ void klibido::slotViewFile( KURL url)
 			this->addWindow(multiFViewer, KMdi::StandardAdd);
 			activateView(currentWindow);
 		
-		} else this->addWindow(multiFViewer, KMdi::StandardAdd);
+		} else addWindow(multiFViewer, KMdi::StandardAdd);
+//		int index = tabWidget()->indexOf(multiFViewer);
+// 		wMenu->insertItem(QString::number(index) + ' ' + multiFViewer->caption(), index, -1);
 	}
 // 	
 	statusBar()->message(url.fileName() + " displayed");
@@ -824,9 +824,6 @@ void klibido::migrateDbs( )
 {
 	//First, open the version db...
 	
-	
-	
-	
 	Db *versionDb=new Db(dbenv, 0);
 	
 	
@@ -932,6 +929,10 @@ void klibido::migrateDbs( )
 				case 2:
 // 					qDebug("Migrating to version 3 of groupDb");
 					tempg->resetSettings();
+					
+				case 3:
+					qDebug("Version 4 of groupDb");
+					tempg->setDeleteOlder(0);
 					break;
 				default:
 					qDebug("Something very very wrong happend with Db versions!");
@@ -945,8 +946,9 @@ void klibido::migrateDbs( )
 // 			tempg->setUnread(25245);
 			data.set_data(tempg->data());
 			data.set_size(tempg->getRecordSize());
+			qDebug("About to save group");
 // 			qDebug("New group size: %d", tempg->getRecordSize());
-			ret=cursor->put(NULL,&data,DB_CURRENT);
+			ret=cursor->put(&key,&data,DB_CURRENT);
 			if (ret != 0)
 				qDebug("Return from cursor put: %s", dbenv->strerror(ret));
 			
@@ -960,6 +962,7 @@ void klibido::migrateDbs( )
 			key.set_flags(DB_DBT_MALLOC);
 			memset(&data, 0, sizeof(data));
 			data.set_flags(DB_DBT_MALLOC);
+			qDebug("End of cycle");
 			
 			
 		}
@@ -996,7 +999,8 @@ void klibido::migrateDbs( )
 	}
 	
 	if (headerDbVersion != HEADERDB_VERSION) {
-		KMessageBox::information(0, i18n("This version of KLibido uses a different format for the header databases. Please press OK and wait while I migrate your DBs (this may take some time...)"), i18n("DB Migration needed"));
+		if (KMessageBox::questionYesNo(0, i18n("This version of KLibido uses a different format for the header databases. The operation can be lengthy. Do you want to continue?"), i18n("DB Migration needed")) == KMessageBox::No)
+			exit(-1);
 		
 		Db* groupDb=new Db(dbenv,0);
 // 		Db* headerDb=new Db(dbenv, 0);
@@ -1065,7 +1069,11 @@ void klibido::migrateDbs( )
 								
 							}
 						}
-						
+					case 3:
+						//Do nothing, just save the articles with fields rearranged...
+					case 4:
+						//do nothing, save with size
+						break;
 				}
 					
 				
@@ -1115,11 +1123,7 @@ void klibido::migrateDbs( )
 			
 			
 		}
-		
-		
-		//For every group in the list...
-				
-		//for every group...
+
 		groupCursor->close();
 		groupDb->close(0);
 		delete groupDb;
@@ -1176,6 +1180,7 @@ void klibido::migrateDbs( )
 			ng->getDb()->cursor(0, &headerCursor, 0); //READ ONLY
 			SIndex *si;
 			//For every record in the group...
+			char *p=NULL;
 			while ( (ret=headerCursor->get(&key, &data, DB_NEXT )) == 0) {
 				
 				bh=new BinHeader((uchar*)data.get_data());
@@ -1191,7 +1196,7 @@ void klibido::migrateDbs( )
 						si->index=bh->getSubj()+bh->getFrom();
 						si->partId=bh->pnmit.key();
 						//Marshall it...
-						char *p= new char[si->index.length() + 2*sizeof(int)];
+						p= new char[si->index.length() + 2*sizeof(int)];
 						char *i = &p[0];
 						i=insert(si->index, i);
 						memcpy(&i, &(si->partId), sizeof(int));
@@ -1236,6 +1241,87 @@ void klibido::migrateDbs( )
 	}
 	
 #endif
+
+	//Migration of the availablegroups db...
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+	
+	data.set_flags(DB_DBT_MALLOC);
+	
+	uint availableGroupsDbVersion;
+	const char *availableGroupsDbKey="AVAILABLEGROUPSDB";
+	key.set_data((char*) availableGroupsDbKey);
+	key.set_size(strlen(availableGroupsDbKey));
+	ret = versionDb->get(0, &key, &data, 0);
+	if (ret == DB_NOTFOUND) {
+		availableGroupsDbVersion = 0;
+	} else if (ret == 0) {
+		memcpy(&availableGroupsDbVersion, data.get_data(), sizeof(uint));
+		free(data.get_data());
+	} else {
+		kdDebug() << "Error retrieving availabegroups version: " << ret << endl;
+		exit(-1);
+	}
+	if (availableGroupsDbVersion != AVAILABLEGROUPSDB_VERSION) {
+		//migrate!
+		kdDebug() << "Need grouplist migration from version " << availableGroupsDbVersion << endl;
+		Dbt aGroupsKey, aGroupsData;
+		
+		Group *g=0;
+		Dbc *aGroupsCursor;
+		Db *aGroupsDb = new Db(dbenv, 0);
+		if (aGroupsDb->open(NULL, "availableGroups", NULL, DB_BTREE, DB_CREATE | DB_THREAD, 0644) != 0) {
+			kdDebug() << "Error opening available groups db\n";
+			exit(-1);
+		}
+		
+		aGroupsDb->cursor(0, &aGroupsCursor, DB_WRITECURSOR);
+		memset(&aGroupsKey, 0, sizeof(aGroupsKey));
+		aGroupsKey.set_flags(DB_DBT_MALLOC);
+		memset(&aGroupsData, 0, sizeof(aGroupsData));
+		aGroupsData.set_flags(DB_DBT_MALLOC);
+		QMap<int, bool>::iterator it;
+		
+		while (aGroupsCursor->get(&aGroupsKey, &aGroupsData, DB_NEXT) == 0) {
+			switch (availableGroupsDbVersion) {
+				case 0:
+					g = new Group((char*)aGroupsData.get_data(), availableGroupsDbVersion);
+					free(aGroupsData.get_data());
+					for (it = g->serverPresence.begin(); it != g->serverPresence.end(); ++it) {
+						g->setArticles(it.key(), 0);
+					}
+				default:
+					break;
+					
+			}
+			//Save the group.
+			char *p = g->data();
+			memset(&aGroupsData, 0, sizeof(aGroupsData));
+			aGroupsData.set_data(p);
+			aGroupsData.set_size(g->size());
+			ret = aGroupsCursor->put(&aGroupsKey, &aGroupsData, DB_CURRENT);
+			if (ret != 0)
+				kdDebug() << "Error putting group:" << ret << endl;
+			delete [] p;
+			delete g;
+			free(data.get_data());
+			free(aGroupsKey.get_data());
+			memset(&aGroupsKey, 0, sizeof(aGroupsKey));
+			aGroupsKey.set_flags(DB_DBT_MALLOC);
+			memset(&aGroupsData, 0, sizeof(aGroupsData));
+			aGroupsData.set_flags(DB_DBT_MALLOC);
+				
+			
+		}
+		aGroupsCursor->close();
+		aGroupsDb->close(0);
+		delete aGroupsDb;
+		
+	}
+
+
+
+
 	memset(&key, 0, sizeof(key));
 	memset(&data, 0, sizeof(data));
 	
@@ -1263,6 +1349,7 @@ void klibido::migrateDbs( )
 	
 	
 	
+	
 	if (serverDbVersion != SERVERDB_VERSION) {
 		kdDebug() << "Need serverDb migration: " << serverDbVersion << " -> " << SERVERDB_VERSION << endl;
 		Dbt serverKey, serverData;
@@ -1271,40 +1358,39 @@ void klibido::migrateDbs( )
 		Db *serverDb = new Db(dbenv, 0);
 		if (serverDb->open(NULL, "servers" , NULL, DB_BTREE, DB_CREATE | DB_THREAD , 0644) != 0)
 			qDebug("Error opening servers db");
-		switch (serverDbVersion) {
-			case 0:
-				//Add tries!
-				serverDb->cursor(0, &serverCursor, DB_WRITECURSOR);
-				memset(&serverKey, 0, sizeof(serverKey));
-				serverKey.set_flags(DB_DBT_MALLOC);
-				memset(&serverData, 0, sizeof(serverData));
-				serverData.set_flags(DB_DBT_MALLOC);
-				while (serverCursor->get(&serverKey, &serverData, DB_NEXT) == 0 ) {
+		serverDb->cursor(0, &serverCursor, DB_WRITECURSOR);
+		memset(&serverKey, 0, sizeof(serverKey));
+		serverKey.set_flags(DB_DBT_MALLOC);
+		memset(&serverData, 0, sizeof(serverData));
+		serverData.set_flags(DB_DBT_MALLOC);
+		char *p=NULL;
+		while (serverCursor->get(&serverKey, &serverData, DB_NEXT) == 0 ) {
+			switch (serverDbVersion) {
+				case 0:
+					//Add tries!
 					//LOAD
 					nh=loadHost((char*)serverData.get_data(), serverDbVersion);
 					free(serverData.get_data());
 					memset(&serverData, 0, sizeof(serverData));
-					
-					//Modify
+						
+						//Modify
 					nh->retries=3;
-					//SAVE
-					char *p=saveHost(nh);
+						//SAVE
+					p=saveHost(nh);
 					serverData.set_data(p);
 					serverData.set_size(::size(nh));
-					serverCursor->put(NULL, &serverData, DB_CURRENT);
+					serverCursor->put(&serverKey, &serverData, DB_CURRENT);
 					free(p);
 					free(serverKey.get_data());
 					memset(&serverKey, 0, sizeof(serverKey));
 					serverKey.set_flags(DB_DBT_MALLOC);
 					memset(&serverData, 0, sizeof(serverData));
 					serverData.set_flags(DB_DBT_MALLOC);
-					
-					
-				}
-				
-				
-				break;
+				default:
+					break;
+			}
 		}
+		serverCursor->close();
 		serverDb->close(0);
 		delete serverDb;
 	}
@@ -1354,6 +1440,17 @@ void klibido::migrateDbs( )
 	if (ret != 0)
 		qDebug("Error writing version db: %d", ret);
 #endif
+	//for availablegroupsdb...
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+	key.set_data((char*) availableGroupsDbKey);
+	key.set_size(strlen(availableGroupsDbKey));
+	version = AVAILABLEGROUPSDB_VERSION;
+	data.set_data(&version);
+	data.set_size(sizeof(version));
+	ret = versionDb->put(0, &key, &data, 0);
+	if (ret != 0)
+		kdDebug() << "Error writing version db: " << ret << endl;
 	
 	//for serversDb...
 	memset(&key, 0, sizeof(key));
@@ -1370,14 +1467,40 @@ void klibido::migrateDbs( )
 	
 	versionDb->close(0);
 	delete versionDb;
+
+}
+QStatusServerWidget::QStatusServerWidget( QString serverName, QWidget * parent, const char * name, WFlags fl ) : QWidget(parent, name, fl)
+{
+	m_sName = serverName;
+	QHBoxLayout *layout = new QHBoxLayout(this);
+	layout->setSpacing(3);
+	iLabel = new QLabel(this);
+	icon = new QPixmap(KGlobal::iconLoader()->loadIcon("icon_server", KIcon::Small, 0, false));
+	iLabel->setPixmap(*icon);
 	
-	
-	
-	
-	
+	sLabel = new QLabel(this);
+	sLabel->setText(m_sName + ": Disconnected");
+	layout->addWidget(iLabel);
+	layout->addWidget(sLabel);
 	
 }
 
+
+void QStatusServerWidget::updateSpeed( int speed )
+{
+	if (speed == -1) {
+		//Idle
+		sLabel->setText(m_sName + ": Idle");
+	} else if (speed == -2) {
+		//Disconnected
+		sLabel->setText(m_sName + ": Disconnected");
+	} else {
+		sLabel->setText(m_sName + ':' + QString::number(speed) + " KB/s");
+		
+	}
+}
+
+
 QToolbarWidget::QToolbarWidget( QWidget * parent, const char * name, WFlags fl ) : QWidget (parent, name, fl)
 {
 	QHBoxLayout *layout = new QHBoxLayout(this);
@@ -1400,7 +1523,7 @@ void QToolbarWidget::slotQueueInfo( unsigned long long  count, unsigned long lon
 	uint i = 1;
 	uint l = sSize.length();
 // 	qDebug("Size length: %d", l);
-	
+	//format the number
 	while ( l > 3*i ) {
 		sSize.insert(sSize.length() - ((3*i) + i - 1), '.');
 		++i;
@@ -1410,14 +1533,6 @@ void QToolbarWidget::slotQueueInfo( unsigned long long  count, unsigned long lon
 	tLabel->setText(QString::number(count) + " items, " + sSize + " KB");
 }
 
-void klibido::slotCloseActiveView( )
-{
-	if (activeWindow()) {
-		activeWindow()->close();
-		
-	}
-		
-}
 
 void klibido::slotShortcutsConfigure( )
 {
@@ -1507,8 +1622,323 @@ void klibido::slotUpdateCurrent( )
 }
 
 
+bool klibido::dbsExists(QString dbDir)
+{
+  QString dir = QDir::cleanDirPath(dbDir);
+
+  if (!QFile::exists(QString("%1/availableGroups").arg(dir)) ||
+      !QFile::exists(QString("%1/newsgroups").arg(dir)) ||
+      !QFile::exists(QString("%1/nzb").arg(dir)) ||
+      !QFile::exists(QString("%1/queue").arg(dir)) ||
+      !QFile::exists(QString("%1/servers").arg(dir)) ||
+      !QFile::exists(QString("%1/versiondb").arg(dir)))
+    return false;
 
 
+  return true;
+}
+
+void klibido::createDbs()
+{
+    qDebug("Create dbs...");
+    //Create the version Db...probably not needed with the new exec path provided by Gonéri, but
+	//it shouldn't do any harm :)
+		if (!dbenv)
+			setupDbEnv(Config().dbDir);
+		else kdDebug() << "slotSettingsConfigure(): Dbenv already opened\n";
+		Db *versionDb=new Db(dbenv, 0);
+		if (versionDb->open(NULL, "versiondb" , NULL, DB_BTREE, DB_CREATE | DB_THREAD , 0644) != 0)
+			qDebug("Error opening versiondb");
+	//Keys, data have fixed size:
+	//DATA is a uint
+	//KEY is one of GROUPDB, AVAILABLEGROUPSDB, HEADERDB
+		Dbt key,data;
+		
+		
+		uint groupDbVersion;
+		groupDbVersion=GROUPDB_VERSION;
+		data.set_data(&groupDbVersion);
+		data.set_size(sizeof(uint));
+			//,availableGroupsDbVersion, headersDbVersion;
+		int ret;
+		
+		//Put groupDb version in versionDb
+		const char *groupDbKey="GROUPDB";
+		key.set_data((char *)groupDbKey);
+		key.set_size(strlen(groupDbKey));
+		ret=versionDb->put(0, &key, &data, 0);
+		if (ret != 0) {
+
+			qDebug("Error putting groupdb version: %s", dbenv->strerror(ret));
+		
+		}
+		
+		//put headerDb version in versionDb
+		memset(&key, 0, sizeof(key));
+// 		key.set_flags(DB_DBT_MALLOC);
+		memset(&data, 0, sizeof(data));
+// 		data.set_flags(DB_DBT_MALLOC);
+		//I could use groupDbVersion, but it's better to create a properly named local
+		//variable to avoid confusion...
+		uint headerDbVersion;
+		headerDbVersion=HEADERDB_VERSION;
+		data.set_data(&headerDbVersion);
+		data.set_size(sizeof(uint));
+	
+		const char *headerDbKey="HEADERDB";
+		key.set_data((char*)headerDbKey);
+		key.set_size(strlen(headerDbKey));
+		ret=versionDb->put(0, &key, &data, 0);
+		if (ret != 0) {
+			qDebug("Error putting headerdb version: %s", dbenv->strerror(ret));
+		}
+		
+		//availablegroupsdbkey...
+		memset(&data, 0, sizeof(data));
+		uint availableGroupsDbVersion = AVAILABLEGROUPSDB_VERSION;
+		data.set_data(&availableGroupsDbVersion);
+		data.set_size(sizeof(uint));
+		const char *aGroupsDbKey="AVAILABLEGROUPSDB";
+		key.set_data((char*)aGroupsDbKey);
+		key.set_size(strlen(aGroupsDbKey));
+		ret=versionDb->put(0, &key, &data, 0);
+		if (ret != 0)
+			kdDebug() << "Error putting agroupsdb version: " << ret << endl;
+		
+		
+		//serverdb key...
+		memset(&key, 0, sizeof(key));
+		memset(&data, 0, sizeof(data));
+		uint serverDbVersion = SERVERDB_VERSION;
+		data.set_data(&serverDbVersion);
+		data.set_size(sizeof(uint));
+		const char *serverDbKey="SERVERDB";
+		key.set_data((char*)serverDbKey);
+		key.set_size(strlen(serverDbKey));
+		ret=versionDb->put(0, &key, &data, 0);
+		if (ret != 0)
+			kdDebug() << "Error putting serverdb version: " << ret << endl;
+		
+		
+		
+		
+		
+		versionDb->close(0);
+		delete versionDb;
+		versionDb=0;
+
+}
+
+void klibido::slotPopupTabMenu( QWidget *w, const QPoint &p )
+{
+//  	qDebug("Widget: %s", (const char *) tabWidget()->label(w));
+// 	KMdiChildView *v = w;
+	int i = tabWidget()->indexOf(w);
+	qDebug("Index: %d", i);
+	KMdiChildView *v = dynamic_cast<KMdiChildView*>(tabWidget()->page(i));
+	if (v)
+		slotPopupWindowMenu(p, v);
+}
+
+void klibido::slotCloseWindow()
+{
+	if (activeWindow()) {
+		activeWindow()->close();
+		
+	}
+
+}
+
+void klibido::slotCloseAndMark(KMdiChildView *v )
+{
+	FileViewer *fv = dynamic_cast<FileViewer*>(v);
+	NewHeaderList *hl = dynamic_cast<NewHeaderList*>(v);
+	if (fv)
+		fv->closeAndDelete();
+	else if (hl)
+		hl->closeAndMark();
+		
+	
+}
+
+
+
+void klibido::slotCloseAndNoMark(KMdiChildView *v )
+{
+	FileViewer *fv = dynamic_cast<FileViewer*>(v);
+	NewHeaderList *hl = dynamic_cast<NewHeaderList*>(v);
+	if (fv)
+		fv->closeNoDelete();
+	else if (hl)
+		hl->closeAndNoMark();
+	
+	
+}
+
+
+
+
+void klibido::slotCloseAll( )
+{
+}
+
+WPopupMenu::WPopupMenu( QWidget *p, QString title, KMdiChildView * hl ) :  KPopupMenu(p), m_hList(hl), m_title(title)
+{
+	insertTitle(m_title);
+	
+	NewHeaderList *hList = dynamic_cast<NewHeaderList*>(m_hList);
+	FileViewer *fViewer = dynamic_cast<FileViewer*>(m_hList);
+	if (hList) {
+		insertItem("Close", this, SLOT(slotCloseWindow()));
+		connect(this, SIGNAL(sigCloseWindow(KMdiChildView* )), p, SLOT(slotCloseWindow(KMdiChildView*)));
+		insertItem(i18n("Close and mark as read"), this, SLOT(slotCloseAndMark()));
+		connect(this, SIGNAL(sigCloseAndMark(KMdiChildView*)), p, SLOT(slotCloseAndMark(KMdiChildView*)));
+		insertItem(i18n("Close and don't mark as read"), this, SLOT(slotCloseAndNoMark()));
+		connect(this, SIGNAL(sigCloseAndNoMark(KMdiChildView*)), p, SLOT(slotCloseAndNoMark(KMdiChildView*)));
+		
+	}  else if (fViewer) {
+		if (!Config().singleViewTab) {
+			insertItem("Close", this, SLOT(slotCloseWindow()));
+			connect(this, SIGNAL(sigCloseWindow(KMdiChildView* )), p, SLOT(slotCloseWindow(KMdiChildView*)));
+			insertItem(i18n("Close and delete"), this, SLOT(slotCloseAndMark()));
+			connect(this, SIGNAL(sigCloseAndMark(KMdiChildView*)), p, SLOT(slotCloseAndMark(KMdiChildView*)));
+			insertItem(i18n("Close and don't delete"), this, SLOT(slotCloseAndNoMark()));
+			connect(this, SIGNAL(sigCloseAndNoMark(KMdiChildView*)), p, SLOT(slotCloseAndNoMark(KMdiChildView*)));
+		}
+	}
+	
+	
+}
+
+void klibido::slotCloseWindow( KMdiChildView *v )
+{
+	if (v)
+		v->close();
+}
+
+void WPopupMenu::slotCloseWindow( )
+{
+	emit sigCloseWindow(m_hList);
+}
+
+
+void WPopupMenu::slotCloseAndMark( )
+{
+	emit sigCloseAndMark(m_hList);
+}
+
+void WPopupMenu::slotCloseAndNoMark( )
+{
+	emit sigCloseAndNoMark(m_hList);
+}
+
+
+void klibido::slotWMenuActivated( int id )
+{
+	qDebug("Activated: %d", id);
+	
+	if (wMenu->idAt(0) != id) {
+		switch (id) {
+			case 1000:
+				slotCloseWindow(activeWindow());
+				break;
+			case 1001:
+				slotCloseAndMark(activeWindow());
+				break;
+			case 1002:
+				slotCloseAndNoMark(activeWindow());
+				break;
+			default:
+				activateView(id);
+				break;
+		}
+	}
+	
+}
+
+/*
+void klibido::childWindowCloseRequest( KMdiChildView * pWnd )
+{
+// 	int index = tabWidget()->indexOf(pWnd);
+// 	wMenu->removeItem(index);
+	KMdiMainFrm::childWindowCloseRequest(pWnd);
+}*/
+
+void klibido::slotAboutToShow( )
+{
+// 	qDebug("About to show");
+// 	for (int i = 1; i < wMenu->count(); i++)
+// 		wMenu->removeItemAt(i);
+	wMenu->clear();
+	closeAction->plug(wMenu);
+	
+	KMdiChildView *view=activeWindow();
+	FileViewer *fv = dynamic_cast<FileViewer*>(view);
+	NewHeaderList *hl = dynamic_cast<NewHeaderList*>(view);
+// 	wMenu->insertItem(KGlobal::iconLoader()->loadIcon("fileclose",KIcon::Toolbar, 0, false), i18n("Close"), 
+// 					  1000, -1);
+// 	wMenu->setAccel(CTRL + Key_W, 1000);
+	if (activeWindow() == qMgr) {
+// 		wMenu->setItemEnabled(1000, false);
+		wMenu->setItemEnabled(wMenu->idAt(0), false);
+		wMenu->insertSeparator();
+	} else if (hl) {
+		
+// 		qDebug("HeaderList active: %s", (const char *) hl->caption());
+		wMenu->insertItem(i18n("Close and mark as read"), 1001, -1);
+		wMenu->insertItem(i18n("Close and don't mark as read"),1002, -1);
+		wMenu->insertSeparator();
+		wMenu->setItemEnabled(wMenu->idAt(0), true);
+	} else if (fv ) {
+// 		qDebug("FileViewer active?");
+		if (!Config().singleViewTab) {
+			wMenu->insertItem(i18n("Close and delete"), 1001, -1);
+			wMenu->insertItem(i18n("Close and don't delete"), 1002, -1);
+			wMenu->setItemEnabled(wMenu->idAt(0), true);
+		} else wMenu->setItemEnabled(wMenu->idAt(0), false);
+		wMenu->insertSeparator();
+	} else {
+		wMenu->setItemEnabled(wMenu->idAt(0), true);
+	}
+// 	wMenu->insertItem("0 Queue manager", 0, -1);
+	KMdiIterator< KMdiChildView * > *wIt =  createIterator ();
+	for (wIt->first(); !wIt->isDone(); wIt->next()) {
+// 		qDebug("Caption: %s", (const char *) wIt->currentItem()->caption());
+		int index = tabWidget()->indexOf(wIt->currentItem());
+		wMenu->insertItem(QString::number(index) + ' ' + wIt->currentItem()->caption(), index, -1);
+		
+	}
+	
+}
+
+void klibido::slotCloseActiveView( )
+{
+
+
+	if (activeWindow())
+		closeWindow(activeWindow());
+}
+
+void klibido::slotUpdateSpeed( int serverId, int speed)
+{
+	sWidgets[serverId]->updateSpeed(speed);
+}
+
+void klibido::slotDeleteServerWidget( int serverId)
+{
+	delete(sWidgets[serverId]);
+	sWidgets.remove(serverId);
+}
+
+void klibido::slotAddServerWidget( NntpHost *nh )
+{
+	//I want the queue widget to be the at the far right...
+	statusBar()->removeWidget(qTWidget);
+	sWidgets[nh->id] = new QStatusServerWidget(nh->name, this);
+	statusBar()->addWidget(sWidgets[nh->id], 0, true);
+	statusBar()->addWidget(qTWidget, 0, true);
+	
+}
 
 
 #include "klibido.moc"
