@@ -145,7 +145,15 @@ int NntpThreadSocket::m_handleError( int expected, QString received )
 
 bool NntpThreadSocket::m_sendCmd( QString& cmd, int response )
 {
-
+	/*
+	//BEGIN DEBUG
+	if (qId == 1) {
+		errorString = "No such article error";
+		error = NntpThreadSocket::NoSuchArticle_Err;
+		return false;
+	}
+	//END DEBUG
+	*/
     QString s;
     error=NntpThreadSocket::No_Err;
 	
@@ -1449,7 +1457,12 @@ void NntpThreadSocket::run( )
 	//While there are jobs in the queue, process jobs :)
 	
 // 	while ( (*status != Paused) && (job=findFreeJob() )) {
+	ce=new NntpThreadEvent(qId, threadId, NntpThreadSocket::StartedWorking);
+	QApplication::postEvent(parent, ce);
+// 	qDebug("%d, %d: sent startedworking message", qId, threadId);
 	while (true) {
+// 		qDebug("qID: %d, tID: %d", qId, threadId);
+		
 		
 		statusLock.lock();
 		
@@ -1501,7 +1514,7 @@ void NntpThreadSocket::run( )
 		
 		
 		ce=new NntpThreadEvent(job, NntpThreadSocket::Start);
-		QApplication::QApplication::postEvent(parent, ce);
+		QApplication::postEvent(parent, ce);
 		
 		switch (job->jobType) {
 			case Job::UpdHead:
@@ -1694,7 +1707,7 @@ void NntpThreadSocket::run( )
 		
 	}
 	//Exited. Why?
-// 	qDebug("about to exit run() on thread %d", threadId);
+	qDebug("about to exit run() on thread %d,%d", qId, threadId);
 // 	statusLock.lock();
 	//I can finally base this on the status *only* - much cleaner
 	
@@ -1712,6 +1725,7 @@ void NntpThreadSocket::run( )
 		case Working:
 			status=Ready;
 			ce = new NntpThreadEvent(NULL, Ready, threadId, qId);
+			//qDebug("%d, %d: sent ready message", qId, threadId);
 			break;
 		default:
 			qDebug("Error: wrong status %d", status);
@@ -2144,10 +2158,15 @@ int NntpThreadSocket::dbGroupPut( Db * db, const char *line, int hostId )
 	QString s=line;
 	QStringList fields=QStringList::split(' ', s, true);
 	QString ngName=fields[0];
+	int articles = fields[1].toInt() - fields[2].toInt();
+	if (articles < 0)
+		articles = 0;
+// 	kdDebug() <<"Articles: " << articles << endl;
 // 	QString desc=fields[1];
 	const char *i= (const char *) ngName;
 	listkey.set_data((void*)i);
 	listkey.set_size(ngName.length());
+// 	kdDebug() << "Articles on " << ngName << ": " << articles << endl;
 	
 // 	memcpy(keymem, i, ngName.length());
 	
@@ -2158,7 +2177,8 @@ int NntpThreadSocket::dbGroupPut( Db * db, const char *line, int hostId )
 	if (ret == DB_NOTFOUND) {
 		
 		//Not found, create new group and insert into db
-		Group *g=new Group(ngName, QString::null , hostId);
+		Group *g=new Group(ngName, QString::null , hostId, articles);
+// 		g->setArticles(hostId, articles);
 		//save into db...
 		char *p=g->data();
 		
@@ -2181,6 +2201,7 @@ int NntpThreadSocket::dbGroupPut( Db * db, const char *line, int hostId )
 		Group *g = new Group((char*)listdata.get_data());
 		free(listdata.get_data());
 		g->addHost(hostId);
+		g->setArticles(hostId, articles);
 		char *p=g->data();
 // 		memcpy(datamem, p, g->size());
 // 		data->set_size(g->size());
