@@ -31,8 +31,14 @@
 #include "mylistview.h"
 
 
+extern bool g_qResume;
+extern bool g_qDiscard;
+
+
 QMgr::QMgr(Servers *_servers, Db *_gdb, QString caption, QWidget* parent) : KMdiChildView(caption,parent), servers(_servers), gdb(_gdb)
 {
+// 	kdDebug() << "KConstructor\n";
+// 	qDebug("QConstructor");
 	nextItemId=0;
 	nextJobId=0;
 	operationsPending=0;
@@ -40,7 +46,6 @@ QMgr::QMgr(Servers *_servers, Db *_gdb, QString caption, QWidget* parent) : KMdi
 	queueCount=0;
 	diskError=0;
 	
-	
 	(new QHBoxLayout(this))->setAutoAdd(true);
 
 	tabWidget=new KTabWidget(this);
@@ -292,6 +297,8 @@ void QMgr::slotAddPostItem( BinHeader * bh, NewsGroup *ng , bool first, bool vie
 	
 	qsi->tmpDir=tempDir;
 	qsi->group=ng->getName();
+	qsi->id = nextItemId;
+	qsi->type = QSaveItem::QSI_Add;
 	
 	qsi->index=bh->getSubj() + bh->getFrom();
 	qsi->rootFName=rootFName;
@@ -393,10 +400,12 @@ void QMgr::slotAddPostItem( BinHeader * bh, NewsGroup *ng , bool first, bool vie
 	queueLock.unlock();
 	emit threadFinished();
 	//Save the item in the queue db...
-	if (!writeSaveItem(nextItemId, qsi))
-		writeError();
+// 	if (!writeSaveItem(nextItemId, qsi))
+// 		writeError();
+// 	delete qsi;
+	writeSaveItem(qsi);
+	
 	
-	delete qsi;
 	
 	
 	
@@ -537,7 +546,7 @@ void QMgr::start( Job * j )
 	queue[j->qItemId]->start(j->partId);
 	queueLock.unlock();
 	threads[j->qId][j->threadId]->started();
-	emit sigThreadStart(j->qId, j->threadId);
+	
 	
 	
 	
@@ -766,7 +775,8 @@ void QMgr::slotThreadFinished( )
 	
 	for (it=queueItems.begin(); it != queueItems.end(); ++it) {
 // 		qDebug("thread finished, processing q...");
-		emit newItem(it.key()); //This calls slotProcessQueue.
+		if (!queueItems[it.key()].isEmpty())
+			emit newItem(it.key()); //This calls slotProcessQueue.
 	}
 // 	qDebug("Exiting QMgr::slotThreadFinished()");
 	
@@ -990,10 +1000,14 @@ int QMgr::findIndex( int qId, int itemAfter, bool up)
 
 QMgr::~QMgr()
 {
-	
+	if (saveQThread->running())
+		saveQThread->wait();
+	delete saveQThread;
 	qDb->close(0);
 	delete qDb;
 	
+	delete nzbGroup;
+	
 	ThreadMap::iterator tit;
 	QueueThreads::iterator qit;
 	for (qit=threads.begin(); qit != threads.end(); ++qit) {
@@ -1010,6 +1024,8 @@ QMgr::~QMgr()
 		
 	}
 	threads.clear();
+	if (decodeThread)
+		delete decodeThread;
 	
 	
 	
@@ -1020,8 +1036,23 @@ void QMgr::customEvent( QCustomEvent * e)
 	NntpThreadEvent *nte;
     DecodeThreadEvent *dte;
 	UpdateDbThreadEvent *ude;
+	SaveQThreadEvent *sqe;
 	
 	switch (e->type()) {
+		case SAVEQTHREADEVENT:
+			sqe = (SaveQThreadEvent*) e;
+			switch (sqe->event) {
+				case SaveQThreadEvent::SQT_DelNZB:
+					qDebug("Delete nzb");
+					delNzbItem(sqe->nzbIndex);
+					break;
+				case SaveQThreadEvent::SQT_Error:
+					kdDebug() << "Error saving queue!\n";
+					qDebug("Error saving queue!");
+					writeError(0);
+					break;
+			}
+			break;
 		case DECODETHREADEVENT:
 			dte=(DecodeThreadEvent*) e;
 			if (dte->diskErr)
@@ -1116,7 +1147,9 @@ void QMgr::customEvent( QCustomEvent * e)
 // 					qDebug("Paused on error");
 					paused(nte->m_totalLines, nte->m_lines, true);
 					break;
-						
+				case NntpThreadSocket::StartedWorking:
+					started(nte->m_totalLines, nte->m_partialLines);						
+					break;
 				case NntpThreadSocket::Start:
 					
 					start(nte->job);
@@ -1243,7 +1276,7 @@ void QMgr::slotCancelSelected( )
 			
 			part=pit.data();
 			
-			if (part->job->status==Job::Queued_Job) {
+			if (part->job->status==Job::Queued_Job || part->job->status == Job::Paused_Job) {
 				part->job->status=Job::Canceled_Job;
 				if (part->job->jobType == Job::GetPost)
 					(*servers)[part->job->qId]->size-=part->job->artSize;
@@ -1391,7 +1424,7 @@ void QMgr::moveItem( QItem *item )
 // 	it=queueView.find(item->qItemId);
 	queueView.remove(item->qItemId);
 	
-	KListViewItem *temp, *child, *after;
+	KListViewItem *temp = NULL, *child = NULL, *after = NULL;
 	if ( item->failedParts == 0)
 		temp = new KListViewItem(doneList, doneList->lastChild());
 	else temp=new KListViewItem(failedList, failedList->lastChild());
@@ -1483,8 +1516,8 @@ void QMgr::requeue( Job * j )
 	} //else qDebug("Other items in this q, keeping...");
 	
 	newServerId=chooseServer(item->post, j->partId );
-// 	qDebug("New serverId: %d", newServerId);
-	kdDebug() << "new server: " << newServerId << endl;
+	qDebug("New serverId: %d", newServerId);
+// 	kdDebug() << "new server: " << newServerId << endl;
 	
 	//Find the index for the new item....to do so, we:
 	//First, check if there are other jobs for this item in the server's q.
@@ -1495,21 +1528,26 @@ void QMgr::requeue( Job * j )
 	//Very boring, silly, and convoluted. But should work...
 	if (item->queuePresence[newServerId] == false ) {
 		//Gotta add the item to the server's queue, possibly in the right place...
-		kdDebug() << "Presence on new server: false\n";
+// 		kdDebug() << "Presence on new server: false\n";
+		qDebug("Presence on new server: false");
 		
 		QValueList<int>::iterator it=queueView.find(item->qItemId);
 		while (it != queueView.begin()) {
 // 			qDebug("Not first");
+			kdDebug() << "Finding in queueView?\n";
 			--it;
+			kdDebug() << "Looking for " << *it << endl;
 			if ( queueItems[newServerId].contains(*it) ) {
 // 				//Ok, got the item. Now find index and insert the item
 				found=true;
+				kdDebug() << "Found in queueview\n";
 				
 // 				newIndex=findLastJob(newServerId, queue[*it]);
 				//Insert the item in the queueItems
 				//Dangerous?
-				it=queueItems[j->qId].find(*it);
-				queueItems[newServerId].insert(it, 1, item->qItemId);
+				
+				it=queueItems[newServerId].find(*it);
+				queueItems[newServerId].insert(it, item->qItemId);
 // 				item->queuePresence[newServerId]=true;			
 				break;
 								
@@ -1518,7 +1556,8 @@ void QMgr::requeue( Job * j )
 		}
 		if (!found) {
 			//insert at the top...
-			kdDebug() << "Inserted at the top of the list\n";
+// 			kdDebug() << "Inserted at the top of the list\n";
+			qDebug("Inserted at the top of the list");
 			queueItems[newServerId].prepend(item->qItemId);
 // 			item->queuePresence[newServerId]=true;
 					
@@ -1532,7 +1571,7 @@ void QMgr::requeue( Job * j )
 		for (qit=queueItems[newServerId].begin(); qit != queueItems[newServerId].end(); ++qit)
 			kdDebug() << "Queue: " << newServerId << " item: " << *qit << endl;
 		
-	}
+	} else kdDebug() << "item " << item->qItemId << "is already in queue " << newServerId << endl;
 	
 	//Now change the job (finally! :)
 	
@@ -1953,10 +1992,13 @@ void QMgr::startAllThreads( int qId )
 
 void QMgr::stopped( int qId, int threadId )
 {
-// 	qDebug("Thread Stopped");
+	qDebug("%d,%d: QMgr::stopped()", qId, threadId);
 	if (threads[qId][threadId]->stop())
 		emit sigThreadFinished(qId, threadId);
-	else emit sigThreadDisconnected(qId, threadId);
+	else {
+		emit sigThreadDisconnected(qId, threadId);
+		qDebug("%d,%d disconnected", qId, threadId);
+	}
 	
 
 	
@@ -2004,7 +2046,7 @@ void QMgr::slotResumeThread( int qId, int threadId)
 	threads[qId][threadId]->resume();
 	slotResumeThreads();
 }
-
+/*
 char * QMgr::insert( QString s, char * p )
 {
 	int strlen=s.length();
@@ -2015,7 +2057,7 @@ char * QMgr::insert( QString s, char * p )
 	memcpy(p, (const char *)s, strlen);
 	p+=strlen;
 	return p;
-}
+}*/
 
 char * QMgr::retrieve( char * i, QString & s )
 {
@@ -2030,7 +2072,7 @@ char * QMgr::retrieve( char * i, QString & s )
 	memcpy(temp, i, strlen);
 	temp[strlen]='\0';
 	s=temp;
-	delete temp;
+	delete [] temp;
 	i+=strlen;
 	return i;
 
@@ -2074,112 +2116,30 @@ QSaveItem * QMgr::loadSaveItem( char *data )
 }
 
 
-char * QMgr::saveItemData( QSaveItem * si)
-{
-	char *p=new char[saveItemSize(si)];
-	char *i=&p[0];
-	i=insert( si->group, i);
-	i=insert( si->index, i);
-	
-	i=insert( si->rootFName, i);
-	i=insert( si->savePath, i);
-	i=insert( si->tmpDir, i);
-	
-	
-	
-	int szInt=sizeof(uint);
-	memcpy(i, &(si->curPostLines), szInt);
-	i+=szInt;
-	
-	//Now save the partMap...
-	int count = si->partStatus.count();
-	memcpy(i, &count, szInt);
-	i+=szInt;
-	
-	int id, status;
-	QMap<int, int>::iterator it;
-	for (it = si->partStatus.begin(); it != si->partStatus.end() ; ++it) {
-		
-		id = it.key();
-		status = it.data();
-		memcpy(i, &id, szInt);
-		i+=szInt;
-		memcpy(i, &status, szInt);
-		i+=szInt;
-		
-	}
-	
-	return p;
-	
-}
 
-uint QMgr::saveItemSize( QSaveItem *si )
-{
-	return si->group.length() + si->index.length() +  + si->rootFName.length() + \
-			 si->savePath.length() + si->tmpDir.length() + 7*sizeof(uint) \
-			+ 2*(si->partStatus.count())*sizeof(int) ;
-}
 
-bool QMgr::writeSaveItem( int id, QSaveItem * si )
+void QMgr::writeSaveItem(QSaveItem * si )
 {
-	//writes save item si with key id int the queue db
-	Dbt key, data;
-	
-	key.set_data(&id);
-	key.set_size(sizeof(id));
-	data.set_data(saveItemData(si));
-	data.set_size(saveItemSize(si));
-	//TODO:free the data?
-	
-
-	if ( (qDb->put(0, &key, &data, 0) ) != 0) {
-		return false;
-	} else {
-		qDb->sync(0);
-		return true;
-	}
-	
+	sQLock.lock();
+	saveQueue.append(si);
+	sQLock.unlock();
+	if (!saveQThread->running())
+		saveQThread->start();
 	
 	
 }
 
-QSaveItem * QMgr::readSaveItem( int id )
-{
-	Dbt key, data;
-	
-	key.set_data(&id);
-	key.set_size(sizeof(id));
-	
-	data.set_flags(DB_DBT_MALLOC);
-// 	key.set_flags(DB_DBT_MALLOC);
-	
-	if ( (qDb->get(0, &key, &data, 0)) != 0) {
-		qDebug("Error getting item from queue db");
-		return 0;
-	} else {
-		QSaveItem *si = loadSaveItem((char*)data.get_data());
-// 		qDebug("SaveItem Loaded");
-		free(data.get_data());
-// 		free(key.get_data());
-		return si;
-	}
-		
-}
 
-bool QMgr::updateSaveItem( int id, int part, int status, uint lines  )
+void QMgr::updateSaveItem( int id, int part, int status, uint lines  )
 {
-	QSaveItem *si = readSaveItem(id);
-
-	if (!si) {
-		qDebug("Error reading the update item %d during update", id);
-		return true;
-	} else {
-		si->partStatus[part]=status;
-		si->curPostLines=lines;
-		writeSaveItem(id, si);
-		delete si;
-		return false;
-	}
+	
+	QSaveItem *si = new QSaveItem;
+	si->id = id;
+	si->modPart = part;
+	si->modStatus = status;
+	si->curPostLines = lines;
+	si->type = QSaveItem::QSI_Update;
+	writeSaveItem(si);
 }
 
 void QMgr::setDbEnv( QString d, DbEnv * dbe, GroupList *gl )
@@ -2190,8 +2150,9 @@ void QMgr::setDbEnv( QString d, DbEnv * dbe, GroupList *gl )
 	//Open the db...
 	qDb=new Db(dbEnv, 0);
 	if (qDb->open(NULL, "queue" , NULL, DB_BTREE, DB_CREATE | DB_THREAD , 0644) != 0)
-		qDebug("Error opening servers db");
+		kdDebug() << "Error opening queue db\n";
 	nzbGroup = new NewsGroup(dbEnv, "nzb", Config().decodeDir);
+	saveQThread = new SaveQThread(&saveQueue, &sQLock, qDb, this);
 	
 	
 }
@@ -2218,29 +2179,48 @@ void QMgr::checkQueue( )
 	bool go = false;
 	while ( ( (ret=cursor->get(&key, &data, DB_NEXT)) ) ==0 ) {
 		//I know, I know, this sucks :)
-		if (go == false) {		
-			switch (KMessageBox::questionYesNo(this, i18n("Found items in the download Queue from the previous session. Do you want to reload them?") , i18n("Reload download Queue?")) ) {
-				case KMessageBox::Yes:
-					go=true;
-					//Do nothing...
-					break;
-				case KMessageBox::No:
-					//cleanup & return
-					cursor->close();
-					uint *count = new uint;
-					qDb->truncate(0,count,0);
-					kdDebug() << "Truncated " << *count << "items from queue db\n";
-					nzbGroup->getDb()->truncate(0, count, 0);
-					kdDebug() << "Truncated " << *count << " items from nzb db\n";
-					
-					delete count;
-					free(key.get_data());
-					free(data.get_data());
-					
-					return;
-					
-					break;
+		if (go == false) {
+			if (g_qResume == 0 && g_qDiscard == 0) {
+				//ask
+				switch (KMessageBox::questionYesNo(this, i18n("Found items in the download Queue from the previous session. Do you want to reload them?") , i18n("Reload download Queue?")) ) {
+					case KMessageBox::Yes:
+						go=true;
+						//Do nothing...
+						break;
+					case KMessageBox::No:
+						//cleanup & return
+						cursor->close();
+						uint *count = new uint;
+						qDb->truncate(0,count,0);
+						kdDebug() << "Truncated " << *count << "items from queue db\n";
+						nzbGroup->getDb()->truncate(0, count, 0);
+						kdDebug() << "Truncated " << *count << " items from nzb db\n";
+						
+						delete count;
+						free(key.get_data());
+						free(data.get_data());
+						
+						return;
+						
+						break;
+				}
+			} else if (g_qResume) {
+				go = true;
+			} else if (g_qDiscard) {
+				cursor->close();
+				uint *count = new uint;
+				qDb->truncate(0,count,0);
+				kdDebug() << "Truncated " << *count << "items from queue db\n";
+				nzbGroup->getDb()->truncate(0, count, 0);
+				kdDebug() << "Truncated " << *count << " items from nzb db\n";
+						
+				delete count;
+				free(key.get_data());
+				free(data.get_data());
+						
+				return;
 			}
+				
 		}
 		
 		memcpy(&id, key.get_data(), sizeof(uint));
@@ -2335,28 +2315,13 @@ void QMgr::checkQueue( )
 	
 }
 
-bool QMgr::delSaveItem( int id )
+void QMgr::delSaveItem( int id )
 {
-	QSaveItem *qsi=readSaveItem(id);
-	if (qsi->group == "nzb")
-		delNzbItem(qsi->index);
-	delete qsi;
-	
-	Dbt key;
-	key.set_data(&id);
-	key.set_size(sizeof(id));
-	
-	
-	if (qDb->del(0, &key,0) != 0) {
-		qDebug("Error deleting key from db");
-		return false;
-	} else {
-		qDb->sync(0);
-		return true;
-	}
-// 	else qDebug("Key deleted");
-	
 	
+	QSaveItem *qsi = new QSaveItem;
+	qsi->type = QSaveItem::QSI_Delete;
+	qsi->id = id;
+	writeSaveItem(qsi);	
 }
 
 bool QMgr::delNzbItem( QString index )
@@ -2538,7 +2503,7 @@ void QMgr::slotOpenNzb( )
 	int i=0;
 	
 	QDomDocument doc("nzbfile");
-	BinHeader *bh;
+	BinHeader *bh = NULL;
 	doc.setContent(&nzbFile);
 	nzbFile.close();
 	QFileInfo qf(nzbFilename);
@@ -2680,6 +2645,18 @@ void QMgr::slotAddNzbItem(BinHeader* bh, bool first, QString dDir) {
 	
 }
 
+void QMgr::closeEvent( QCloseEvent * e )
+{
+	
+	e->ignore();
+}
+
+void QMgr::started( int serverID, int threadID )
+{
+	emit sigThreadStart(serverID, threadID);
+	
+}
+
 
 
 
