@@ -413,6 +413,14 @@ bool QPostItem::finished( int partId )
 }
 
 
+void Thread::resetSpeed() {
+	speedIndex = 0;
+	for (int i = 0; i < SPEED_MEAN ; i++) {
+		speedBytes[i][0] = 0;
+		speedBytes[i][1] = 0;
+	}
+	prevTime = QTime::currentTime();
+}
 
 
 Thread::Thread(uint _serverId, uint _threadId, uint to, uint rc, QWidget * p ) : serverId(_serverId), threadId(_threadId), threadTimeout(rc)
@@ -420,17 +428,10 @@ Thread::Thread(uint _serverId, uint _threadId, uint to, uint rc, QWidget * p ) :
 	retryCount=0;
 	timeout=to*1000;
 	threadBytes=new uint;
-	
-
-
-
-	
-	
 	*threadBytes=0;
-	prevBytes=0;
+// 	prevBytes=0;
+	resetSpeed();
 // 	status=NntpThreadSocket::Ready;
-	
-
 	nt=new NntpThreadSocket(p, threadBytes);
 	nt->setId(serverId, threadId);
 	speedTimer=new QTimer(this);
@@ -506,8 +507,25 @@ bool Thread::stop( )
 void Thread::slotSpeedTimeout( )
 {
 	
-	emit sigThreadSpeedChanged(serverId, threadId, (int)((*threadBytes-prevBytes)/1024));
+	curTime = QTime::currentTime();
+	interval = prevTime.msecsTo(curTime);
+	prevTime = curTime;
+	speedBytes[speedIndex][0] = *threadBytes - prevBytes;
 	prevBytes = *threadBytes;
+	speedBytes[speedIndex][1] = interval;
+	bytes = 0;
+	interval = 0;
+	for (int i = 0 ; i < SPEED_MEAN; i++) {
+		bytes += speedBytes[i][0];
+		interval += speedBytes[i][1];
+// 		qDebug("Bytes: %f; interval: %d", bytes, interval);
+		
+	}
+// 	emit sigThreadSpeedChanged(serverId, threadId, (int)((*threadBytes-prevBytes)/1024));
+	emit sigThreadSpeedChanged(serverId, threadId, (int)(((bytes/1024)/(float) (interval/1000))));
+	if (++speedIndex >= SPEED_MEAN)
+		speedIndex = 0;
+	
 	
 }
 
@@ -539,6 +557,7 @@ void Thread::comError( )
 	//Now I'm paused...
 // 	status=NntpThreadSocket::Paused;
 	speedTimer->stop();
+	resetSpeed();
 	idleTimer->stop();
 	
 // 	qDebug("%d, %d: retrycount: %d", serverId, threadId, retryCount);
@@ -568,6 +587,7 @@ void Thread::comError( )
 void Thread::paused( )
 {
 	speedTimer->stop();
+	resetSpeed();
 	
 	idleTimer->start(timeout, true);
 // 	qDebug("Thread Paused");
@@ -618,7 +638,7 @@ bool DecodeThread::decode(QPostItem *postItem )
 	QTime start, end;
 	start = QTime::currentTime();
 	
-	DecodeThreadEvent *dte;
+	DecodeThreadEvent *dte = NULL;
 	UUInitialize();
 	UUSetOption(UUOPT_FAST, 1, NULL);
 	//BEGIN modified for debug
@@ -743,6 +763,7 @@ void Thread::canceled( )
 // 		qDebug("Thread::stop(): ok, thread %d,%d stopped ", serverId, threadId);
 	} // else qDebug("Thread::stop(): thread %d,%d stopped", serverId, threadId);
 	speedTimer->stop();
+	resetSpeed();
 	//The thread is now disconnected...I don't need a timeout timer...
 // 	idleTimer->start(timeout, true);
 // 	status=NntpThreadSocket::Ready;
@@ -761,6 +782,7 @@ void Thread::start( )
 // 	retryTimer->stop();
 	
 // 	retryCount=0;
+// 	qDebug("Start!");
 	nt->tStart();
 	
 // 	speedTimer->start(1000, false);
@@ -1123,6 +1145,7 @@ void Thread::resetRetryCounter( )
 void Thread::started( )
 {
 	speedTimer->start(1000);
+	resetSpeed();
 	idleTimer->stop();
 }
 
@@ -1240,7 +1263,7 @@ bool SelfDecodeThread::decode( QPostItem *postItem )
 	QStringList partList;
 	bool errorFlag=false;
 	bool diskError=false;
-	QString errorString="error(s):";
+	QString errorString=QString::null;
 	QMap<int, Part *>::iterator pit;
 	parts = 0;
 	
@@ -1260,12 +1283,12 @@ bool SelfDecodeThread::decode( QPostItem *postItem )
 				case Decoder::Err_BadCRC:
 					//bad crc in part
 					errorFlag=true;
-					errorString += "Bad part crc;";
+					errorString += "Bad part crc; ";
 					break;
 				case Decoder::Err_MissingPart:
 					//Missing part...right now only activated if cannot read from input
 					errorFlag=true;
-					errorString += "Missing part";
+					errorString +="Missing part; ";
 					break;
 				case Decoder::Err_No:
 					//No error
@@ -1273,7 +1296,7 @@ bool SelfDecodeThread::decode( QPostItem *postItem )
 				case Decoder::Err_SizeMismatch:
 					//Size mismatch between expected size and written bytes
 					errorFlag=true;
-					errorString += "Part size mismatch;";
+					errorString += "Part size mismatch; ";
 					break;
 				case Decoder::Err_Write:
 					//Disk error...disk full?
@@ -1338,6 +1361,7 @@ bool SelfDecodeThread::decode( QPostItem *postItem )
 	if (decoder)
 		delete decoder;
 	QApplication::postEvent(parent, dte);
+	qDebug("Posting decoder event");
 	if (diskError)
 		return false;
 	else return true;
@@ -1374,6 +1398,258 @@ void QUpdItem::comError(int partId) {
 	
 }
 
+SaveQThread::SaveQThread( QValueList< QSaveItem * > *list, QMutex * lock, Db * qDb, QWidget * p )
+{
+	saveQDb = qDb;
+	saveQLock = lock;
+	parent = p;
+	saveQItems = list;
+	item = 0;
+}
+
+void SaveQThread::run( )
+{
+	int ok = 0;
+	while(!saveQItems->isEmpty()) {
+		item=saveQItems->first();
+		if ( item &&  (ok = processItem(item ))) {
+			saveQLock->lock();
+			saveQItems->remove(saveQItems->first());
+			saveQLock->unlock();
+			delete item;
+			item=0;
+		} else if (!ok) {
+			SaveQThreadEvent *e = new SaveQThreadEvent(SaveQThreadEvent::SQT_Error);
+			QApplication::postEvent(parent, e);
+			break;
+		}
+
+	}
+	
+}
+
+bool SaveQThread::writeSaveItem( QSaveItem *si )
+{
+	kdDebug() << "Writing save item " << si->id << endl;
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+	key.set_data(&(si->id));
+	key.set_size(szInt);
+	char *p = saveItemData(si);
+	data.set_data(p);
+	data.set_size(saveItemSize(si));
+	//TODO:free the data?
+	
+
+	if ( (saveQDb->put(0, &key, &data, 0) ) != 0) {
+		delete [] p;
+		return false;
+	} else {
+		delete [] p;
+		saveQDb->sync(0);
+		return true;
+	}
+	
+}
+
+QSaveItem *SaveQThread::readSaveItem( int id )
+{
+	memset(&key, 0, sizeof(key));
+	memset(&data, 0, sizeof(data));
+	key.set_data(&id);
+	key.set_size(szInt);
+	
+	data.set_flags(DB_DBT_MALLOC);
+// 	key.set_flags(DB_DBT_MALLOC);
+	
+	if ( (saveQDb->get(0, &key, &data, 0)) != 0) {
+		qDebug("Error getting item from queue db");
+		return 0;
+	} else {
+		QSaveItem *si = loadSaveItem((char*)data.get_data());
+// 		qDebug("SaveItem Loaded");
+		free(data.get_data());
+// 		free(key.get_data());
+		return si;
+	}
+	
+}
+
+bool SaveQThread::processItem( QSaveItem *qsi )
+{
+	switch (qsi->type) {
+		case QSaveItem::QSI_Add:
+			return writeSaveItem(qsi);
+			break;
+		case QSaveItem::QSI_Delete:
+			return delSaveItem(qsi->id);
+			break;
+		case QSaveItem::QSI_Update:
+			return updateItem(qsi);
+			break;
+		default:
+			qDebug("Error: wrong QSaveItem type");
+			return false;
+			break;
+	}
+}
+
+bool SaveQThread::updateItem( QSaveItem *qsi )
+{
+	QSaveItem *si = readSaveItem(qsi->id);
+	kdDebug() << "Updating saveitem " << qsi->id << endl;
+	if (!si) {
+		kdDebug() << "Error reading the save item during update" << qsi->id << endl;
+		return false;
+	} else {
+		si->partStatus[qsi->modPart]=qsi->modStatus;
+		si->curPostLines=qsi->curPostLines;
+		si->id = qsi->id;
+		writeSaveItem(si);
+		delete si;
+		return true;
+	}
+	
+	
+}
+
+bool SaveQThread::delSaveItem( int id )
+{
+	kdDebug() << "Deleting saveItem " << id << endl;
+	QSaveItem *si = readSaveItem(id);
+	if (si->group == "nzb"){
+		SaveQThreadEvent *e = new SaveQThreadEvent(SaveQThreadEvent::SQT_DelNZB, si->index);
+		QApplication::postEvent(parent, e);
+	}
+	delete si;
+		
+	memset(&key, 0, sizeof(key));
+	key.set_data(&id);
+	key.set_size(szInt);
+	
+	
+	if (saveQDb->del(0, &key,0) != 0) {
+		qDebug("Error deleting key from db");
+		return false;
+	} else {
+		saveQDb->sync(0);
+		return true;
+	}
+	
+}
+
+int SaveQThread::saveItemSize( QSaveItem *si )
+{
+	return si->group.length() + si->index.length() +  + si->rootFName.length() + \
+			si->savePath.length() + si->tmpDir.length() + 7*sizeof(uint) \
+			+ 2*(si->partStatus.count())*sizeof(int) ;
+}
+
+char* SaveQThread::saveItemData( QSaveItem *si )
+{
+	char *p=new char[saveItemSize(si)];
+	char *i=&p[0];
+	i=insert( si->group, i);
+	i=insert( si->index, i);
+	
+	i=insert( si->rootFName, i);
+	i=insert( si->savePath, i);
+	i=insert( si->tmpDir, i);
+	
+	
+	
+	int szInt=sizeof(uint);
+	memcpy(i, &(si->curPostLines), szInt);
+	i+=szInt;
+	
+	//Now save the partMap...
+	int count = si->partStatus.count();
+	memcpy(i, &count, szInt);
+	i+=szInt;
+	
+	int id, status;
+	QMap<int, int>::iterator it;
+	for (it = si->partStatus.begin(); it != si->partStatus.end() ; ++it) {
+		
+		id = it.key();
+		status = it.data();
+		memcpy(i, &id, szInt);
+		i+=szInt;
+		memcpy(i, &status, szInt);
+		i+=szInt;
+		
+	}
+	
+	return p;
+}
+
+QSaveItem * SaveQThread::loadSaveItem( char *data )
+{
+	char *i=&data[0];
+	QSaveItem * si = new QSaveItem;
+	i=retrieve(i, si->group);
+	i=retrieve(i, si->index);
+
+	i=retrieve(i, si->rootFName);
+	i=retrieve(i, si->savePath);
+	i=retrieve(i, si->tmpDir);
+	
+	int szInt=sizeof(uint);
+	
+	memcpy( &(si->curPostLines), i, szInt);
+	i+=szInt;
+	
+	int count, id, status;
+	
+	
+	memcpy(&count, i, szInt);
+	i+=szInt;
+	for (int j = 0; j < count ; j++) {
+		memcpy(&id, i, szInt);
+		i+=szInt;
+		memcpy(&status, i, szInt);
+		i+=szInt;
+		si->partStatus.insert(id, status);
+		
+	}
+	
+
+	return si;
+	
+}
+
+char * SaveQThread::insert( QString s, char * p )
+{
+	int strlen=s.length();
+	int suint = sizeof(strlen);
+	memcpy(p, &strlen, suint);
+
+	p+=suint;
+	memcpy(p, (const char *)s, strlen);
+	p+=strlen;
+	return p;
+}
+
+char * SaveQThread::retrieve( char * i, QString & s )
+{
+	int strlen;
+	int ssize=sizeof(strlen);
+
+	char *temp;
+	memcpy(&strlen, i, ssize);
+
+	i+=ssize;
+	temp=new char[strlen+1];
+	memcpy(temp, i, strlen);
+	temp[strlen]='\0';
+	s=temp;
+	delete [] temp;
+	i+=strlen;
+	return i;
+}
+
+
+
 
 
 
