@@ -30,6 +30,8 @@ ServersList::ServersList(QString dbName, DbEnv *_dbEnv, Servers *_servers, QWidg
 	addServers(); //this populates the listview
 	connect(m_serversList, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&,int)), this, SLOT(slotServersPopup(QListViewItem*, const QPoint& )));
 	connect(m_serversList, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
+	serverSpeedTimer = new QTimer(this);
+	connect(serverSpeedTimer, SIGNAL(timeout()), this, SLOT(slotServerSpeedTimeout()));
 	
 }
 
@@ -44,6 +46,8 @@ ServersList::ServersList(Servers *_servers, QWidget * parent, const char * name,
 	serverDb=0;
 	connect(m_serversList, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&,int)), this, SLOT(slotServersPopup(QListViewItem*, const QPoint& )));
 	connect(m_serversList, SIGNAL(selectionChanged()), this, SLOT(slotSelectionChanged()));
+	serverSpeedTimer = new QTimer(this);
+	connect(serverSpeedTimer, SIGNAL(timeout()), this, SLOT(slotServerSpeedTimeout()));
 	
 }
 
@@ -55,12 +59,21 @@ ServersList::~ServersList()
 // 	for (it=servers->begin(); it!=servers->end(); ++it) {
 // 		delete it.data();
 // 	}
+	delete serverSpeedTimer;
 	if (serverDb) {
 		serverDb->close(0);
 		delete serverDb;
 // 		qDebug("Server db closed");
 	}
 	
+	Servers::iterator it;
+	for (it = servers->begin(); it != servers->end(); ++it) {
+		delete it.data();
+		
+	}
+	
+	
+	
 }
 
 void ServersList::addServers( )
@@ -80,10 +93,20 @@ void ServersList::slotSpeedChanged( int serverId, int threadId, int speed )
 	//alternate way...
 	SpeedThread *sp=serverThreads[serverId][threadId];
 	
-	sp->speed=(sp->speed + speed) /2;
+// 	sp->speed=(sp->speed + speed) /2;
+	sp->speed = speed;
 	sp->item->setText(SpeedCol, QString::number( sp->speed ) + " KB/s");
 	
 	
+	
+	
+// 	qDebug("Server speed: %d", serverSpeed);
+	
+// 	(*servers)[serverId]->item->setText(SpeedCol, QString::number(serverSpeed) + "KB/s");
+	
+	
+	
+	
 	/*	
 	(sp->speed)+=speed;
 	sp->item->setText(SpeedCol, QString::number((sp->speed)/(sp->count)) + " KB/s");
@@ -99,7 +122,12 @@ void ServersList::slotThreadStart( int serverId, int threadId)
 	sp->item->setText(StatusCol, "Working...");
 	sp->item->setText(SpeedCol, "0 KB/s");
 	sp->status=Running;
+// 	qDebug("WorkingThread: %d", (*servers)[serverId]->workingThreads);
+// 	qDebug("WorkingThread: %d", (*servers)[serverId]->workingThreads);
 	(*servers)[serverId]->workingThreads++;
+	qDebug("%d, %d: slotThreadStart(); workingThreads: %d", serverId, threadId, (*servers)[serverId]->workingThreads);
+	if (!serverSpeedTimer->isActive())
+		serverSpeedTimer->start(SPEED_TIMEOUT);
 
 	
 // 	serverThreads[serverId][threadId]->setText(StatusCol, "Working...");
@@ -108,12 +136,19 @@ void ServersList::slotThreadStart( int serverId, int threadId)
 
 void ServersList::slotThreadStop( int serverId, int threadId)
 {
+	qDebug("%d, %d: SlotThreadStop", serverId, threadId);
 	SpeedThread *sp=serverThreads[serverId][threadId];
 	sp->item->setText(StatusCol, "Idle");
 	sp->item->setText(SpeedCol, "");
 	sp->count=1;
 	sp->speed=0;
-	sp->status=Stopped;
+	sp->status=Idle;
+	((*servers)[serverId]->workingThreads)--;
+	if ((*servers)[serverId]->workingThreads == 0) {
+		(*servers)[serverId]->item->setText(SpeedCol, "");
+		emit sigServerSpeed(serverId, -1);
+	}
+	
 	
 // 	serverThreads[serverId][threadId]->setText(StatusCol, "Idle");
 // 	serverThreads[serverId][threadId]->setText(SpeedCol, "");
@@ -129,6 +164,11 @@ void ServersList::slotThreadPaused( int serverId, int threadId , int seconds)
 	sp->count=1;
 	sp->speed=0;
 	sp->status=Paused;
+	(*servers)[serverId]->workingThreads--;
+	if ((*servers)[serverId]->workingThreads == 0) {
+		(*servers)[serverId]->item->setText(SpeedCol, "");
+		emit sigServerSpeed(serverId, -1);
+	}
 	
 }
 
@@ -140,23 +180,40 @@ void ServersList::slotCountDown( int serverId, int threadId, int seconds)
 
 void ServersList::slotCanceling( int serverId, int threadId)
 {
+	qDebug("slotCanceling()");
 	SpeedThread *sp=serverThreads[serverId][threadId];
 	sp->item->setText(StatusCol, "Canceling...");
 	sp->item->setText(SpeedCol, "");
 	sp->count=1;
 	sp->speed=0;
+	sp->status=Canceling;
 // 	serverThreads[serverId][threadId]->setText(StatusCol, "Canceling...");
 // 	serverThreads[serverId][threadId]->setText(SpeedCol, "");
 }
 
 void ServersList::slotThreadDisconnect( int serverId , int threadId)
 {
+	qDebug("%d,%d: SlotThreadDisconnected()",serverId, threadId );
 	SpeedThread *sp=serverThreads[serverId][threadId];
 	sp->item->setText(StatusCol, "Disconnected");
 	sp->item->setText(SpeedCol, "");
 	sp->count=1;
 	sp->speed=0;
-	sp->status=Canceling;
+// 	sp->status=Canceling;
+	qDebug("Status: %d", sp->status);
+	qDebug("WorkingThreads: %d", (*servers)[serverId]->workingThreads);
+	if (sp->status != Idle && sp->status != Stopped && sp->status != Paused) {
+		qDebug("Decrementing thread count");
+		((*servers)[serverId]->workingThreads)--;
+		sp->status = Stopped;
+	}
+	//qDebug("WorkingThreads: %d", (*servers)[threadId]->workingThreads);
+	if ((*servers)[serverId]->workingThreads == 0) {
+// 			serverSpeedTimer->stop();
+			(*servers)[serverId]->item->setText(SpeedCol, "");
+			emit sigServerSpeed(serverId, -2);
+	}
+	
 // 	serverThreads[serverId][threadId]->setText(StatusCol, "Disconnected");
 // 	serverThreads[serverId][threadId]->setText(SpeedCol, "");
 }
@@ -347,6 +404,7 @@ void ServersList::m_loadServers( )
 		nh->enabled=true;
 		
 	}
+	
 	cursor->close();
 }
 
@@ -405,7 +463,7 @@ void ServersList::slotDeleteServer( )
 	//4)
 	//Delete the server from the 
 	// - servers' list, container and db.
-	
+	qDebug("phase 4");
 	delete selected;
 	delete (*servers)[serverId];
 	servers->remove(serverId);
@@ -612,6 +670,9 @@ void ServersList::slotDeleteThread( )
 
 void ServersList::slotThreadDeleted( int serverId , int threadId)
 {
+	if (serverThreads[serverId][threadId]->status == Running) {
+		(*servers)[serverId]->workingThreads--;
+	}
 	delete serverThreads[serverId][threadId]->item;
 	delete serverThreads[serverId][threadId];
 	serverThreads[serverId].remove(threadId);
@@ -683,6 +744,7 @@ void ServersList::addThread( QListViewItem* selected,  int serverId)
 // 	newSt->item->setText(1, "Disconnected");
 // 	serverThreads[serverId].insert(threadId, newThreadView);
 	serverThreads[serverId].insert(threadId, newSt);
+	newSt->status = Stopped;
 	
 	//Add to the queueThreads
 	emit sigAddThread(serverId, threadId);
@@ -720,6 +782,35 @@ void ServersList::checkServers( )
 	else emit sigHaveServers(true);
 }
 
+void ServersList::slotServerSpeedTimeout( )
+{	
+// 	kdDebug() << "ServersList::slotServerSpeedTimeout( )\n";
+	Servers::iterator sit;
+	ThreadView::iterator it;
+	int serverSpeed;
+	bool oneIsWorking = false;
+	for (sit = servers->begin(); sit != servers->end(); ++sit) {
+		if (sit.data()->workingThreads > 0) {
+			oneIsWorking = true;
+			ThreadView tv = serverThreads[sit.key()];
+			serverSpeed = 0;
+			for (it = tv.begin(); it != tv.end() ; ++it) {
+				serverSpeed += it.data()->speed;
+			}
+			serverSpeed = (serverSpeed+sit.data()->speed)/2;
+			sit.data()->item->setText(SpeedCol, QString::number( serverSpeed ) + " KB/s");
+			sit.data()->speed=serverSpeed;
+			emit sigServerSpeed(sit.key(), serverSpeed);
+		} //else kdDebug() << "No working threads in server " << sit.key() << endl;
+	}
+	if (!oneIsWorking)
+		serverSpeedTimer->stop();
+	
+			
+	
+}
+
+
 
 
 
