@@ -67,7 +67,7 @@ void UpdateDbThread::run() {
 bool UpdateDbThread::expire( NewsGroup * ng, int hostId) {
     Dbc *cursor;
 	int lw=ng->low[hostId];
-	kdDebug() << "lowWatermark: " << lw << endl;
+	kdDebug() << ng->ngName << " lowWatermark: " << lw << endl;
 	QTime previous, current;
 	previous=QTime::currentTime();
 	uint progress=groupArticles;
